<?php
# php-cmdb - a php based configuration management database

# Copyright (C) 2008  Dr. Michael Scheibe - michael@nofiles.de

# php-cmdb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# php-cmdb is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with php-cmdb.  If not, see <http://www.gnu.org/licenses/>.

/**
* @author Dr. Michael Scheibe
* @package core
* 
* $Id: core_api.inc.php 439 2010-11-12 16:15:08Z micha $
*/

require_once 'core/core_exception.inc.php';
require_once 'core/core_db.inc.php';
require_once 'core/logger.inc.php';

/**
 * get value for property
 * 
 * search for the property in this order:
 * - in db-table property with user
 * - in db-table property for all users
 * - in <base_dir>/config.inc.php
 * - in <base_dir>/includes/config_default.inc.php
 * - the default value from p_default
 * 
 * @param	string		$_prop_name			name of property
 * @param 	string		$_default			default value, if not found
 * @return	mixed
 */
function get_property ( $_prop_name, $_default=null )
{
	// search in db
	$value = get_property_from_db( $_prop_name );
	if (strlen($value)!=0) return $value;
	
	// search in config
	$value = get_property_from_config( $_prop_name );
//	if (strlen($value)!=0) return $value;
	if ($value!=null) return $value;
	
	// return default
	return $_default;
}

/**
 * get user specific property, searched from following sources: database, config 
 * 
 * @param	string		$_prop_name			name of property
 * @param	int			$_type_id			ID of type (0 if not required)
 * @param	int			$_item_id			ID of item (0 if not required)
 * @param	string		$_default			default value, if not found
 * @return	mixed
 */
function get_user_property ( $_prop_name, $_type_id=0, $_item_id=0, $_default=null )
{
	// search in db
	$value = get_property_from_db( $_prop_name, $_type_id, $_item_id );
	if (strlen($value)!=0) return $value;
	
	// search in config
	$value = get_property_from_config( $_prop_name );
	if (strlen($value)!=0) return $value;
	
	// return default
	return $_default;
}

/**
 * recursively search for an entry in db-table property: for user
 * 
 * @param	string		$_prop_name			name of property
 * @param	int			$_type_id			ID of type (0 if not required)
 * @param	int			$_item_id			ID of item (0 if not required)
 * @return	mixed
 */
function get_property_from_db ( $_prop_name, $_type_id=0, $_item_id=0 )
{
	$value = "";
	$user_id = core_session::getInstance()->get_user_id();
	
	// search for user
	if ($user_id != 0)
	{
		$value = select_property_from_db( $_prop_name, $user_id, $_type_id, $_item_id );
		if (strlen($value)!=0) return $value;
	}

	// search without user
	$value = select_property_from_db( $_prop_name, 0, $_type_id, $_item_id );
	if (strlen($value)!=0) return $value;
	
	return $value;
}

/**
 * search property in GLOBALS set from config_default.inc.php 
 * and overloaded from config.inc.php
 * 
 * @param	string		$_prop_name			name of property
 * @return	mixed
 */
function get_property_from_config ( $_prop_name )
{
//	$value = "";
	$value = null;
	if ( isset( $GLOBALS['g_' . $_prop_name] ) ) $value = $GLOBALS['g_' . $_prop_name]; 
	return $value;
}

function set_global_property ( $_prop_name, $_value, $_type_id=0, $_item_id=0 )
{
	if (! insert_property_in_db( $_prop_name, 0, $_type_id, $_item_id, $_value ))
	{
		update_property_in_db( $_prop_name, 0, $_type_id, $_item_id, $_value );
	}
	return;
}

function delete_global_property ( $_prop_name, $_type_id=0, $_item_id=0 )
{
	delete_property_in_db( $_prop_name, 0, $_type_id, $_item_id );
	return;
}

function set_user_property ( $_prop_name, $_value, $_type_id=0, $_item_id=0 )
{
	$user_id = core_session::getInstance()->get_user_id();
	if ($user_id == 0) return;

	if (! insert_property_in_db( $_prop_name, $user_id, $_type_id, $_item_id, $_value ))
	{
		update_property_in_db( $_prop_name, $user_id, $_type_id, $_item_id, $_value );
	}
	
	return;
}

function delete_user_property ( $_prop_name, $_type_id=0, $_item_id=0 )
{
	$user_id = core_session::getInstance()->get_user_id();
	if ($user_id == 0) return;

	delete_property_in_db( $_prop_name, $user_id, $_type_id, $_item_id );

	return;
}

/**
 *  search for an entry in db-table property with the given specification and return
 *  the p_value or empty string if not found
 */
function select_property_from_db ( $_prop_name, $_user_id=0, $_type_id=0, $_item_id=0 )
{
	$value = "";
	try
	{
		$db = core_db::getInstance();
		$query = "SELECT p_value FROM property " .
			"WHERE p_id='".$db->escape_string($_prop_name)."' " .
			"AND p_user_id=$_user_id " .
			"AND p_type_id=$_type_id " .
			"AND p_item_id=$_item_id";
		$result = $db->query( $query );
		if ( ($db->errno($result)!=0) || ($db->num_rows( $result )!=1) ) return $value;
		$value = $db->result( $result, 0 );
	}
	catch (core_db_exception $e) 
	{
	}
	return $value;
}

function insert_property_in_db ( $_prop_name, $_user_id, $_type_id, $_item_id, $_value )
{
	try
	{
		$db = core_db::getInstance();
		$query = "INSERT INTO property " .
			"(p_id, p_user_id, p_type_id, p_item_id, p_value) VALUES ('".
			$db->escape_string($_prop_name)."',".
			"$_user_id,".
			"$_type_id,".
			"$_item_id,".
			"'" . $db->escape_string($_value) . "')";
		$result = $db->query( $query );
	}
	catch (core_db_exception $e) 
	{
		return FALSE;
	}
	
	return TRUE;
}

function update_property_in_db ( $_prop_name, $_user_id, $_type_id, $_item_id, $_value )
{
	try
	{
		$db = core_db::getInstance();
		$query = "UPDATE property SET " .
			"p_value='".$db->escape_string($_value)."' " .
			"WHERE p_id='".$db->escape_string($_prop_name)."' " .
			"AND p_user_id=$_user_id " .
			"AND p_type_id=$_type_id " .
			"AND p_item_id=$_item_id " .
			"LIMIT 1";
		$result = $db->query( $query );
	}
	catch (core_db_exception $e) 
	{
		return FALSE;
	}
	
	return TRUE;
}

function delete_property_in_db ( $_prop_name, $_user_id, $_type_id, $_item_id )
{
	try
	{
		$db = core_db::getInstance();
		$query = "DELETE FROM property " .
			"WHERE p_id='".$db->escape_string($_prop_name)."' " .
			"AND p_user_id=$_user_id " .
			"AND p_type_id=$_type_id " .
			"AND p_item_id=$_item_id " .
			"LIMIT 1";
		$result = $db->query( $query );
	}
	catch (core_db_exception $e) 
	{
		return FALSE;
	}
	
	return TRUE;
}

function delete_all_property_in_db ( $_user_id=0 )
{
	try
	{
		$db = core_db::getInstance();
		$query = "DELETE FROM property " .
			"WHERE p_user_id='".$db->escape_string($_user_id)."' ";
		$result = $db->query( $query );
	}
	catch (core_db_exception $e) 
	{
		return FALSE;
	}
	
	return TRUE;
}

/**
 * get string for given name from string_setup
 * 
 * @param	string	$s_name					Name of the string
 * @return	string
 */
function get_string( $s_name )
{
	$key = 's_' . $s_name;
	if (! in_array( $key, $GLOBALS) )
		trigger_error( "STRING $s_name NOT SET", E_USER_ERROR );

	$txt = $GLOBALS[ $key ];
	if (strlen($txt)==0)
//		$txt = "STRING $s_name IS EMPTY";
		$txt = "*$s_name*";

	return $txt;
}

/**
 * explode_enum_string
 *
 * Breaks up an enum string into num:value elements 
 */
function explode_enum_string( $enum_string_name ) {
	return explode( ',', get_string( $enum_string_name ) );
}

/**
 * explode_enum_arr
 * 
 * Given one num:value pair it will return both in an array
 * num will be first (element 0) value second (element 1)
 */
function explode_enum_arr( $p_enum_elem ) {
	return explode( ':', $p_enum_elem );
}

/**
 * get_enum_to_array
 * 
 * Get the string associated with the $p_enum value
 */
function get_enum_to_array( $enum_string_name ) {
	$t_arr = explode_enum_string( $enum_string_name );
	$enum_count = count( $t_arr );
	for ($i=0; $i < $enum_count;$i++) {
		$t_s = explode_enum_arr( $t_arr[$i] );
		$t_index = (int) $t_s[0];
		$t_array[$t_index] = $t_s[1];
	}
	return $t_array;
}

/**
 * get_enum_to_string
 * 
 * Get the string associated with the $p_enum value
 */
function get_enum_to_string( $enum_string_name, $p_num ) {
	$t_arr = explode_enum_string( $enum_string_name );
	$enum_count = count( $t_arr );
	for ($i=0; $i < $enum_count;$i++) {
		$t_s = explode_enum_arr( $t_arr[$i] );
		if ( $t_s[0] == $p_num ) {
			return $t_s[1];
		}
	}
	return '@' . $p_num . '@';
}

/**
 * liefert den page-namen aus der url
 * 
 * @param	$url		URL
 * @return	string		Page-Name
 */

function get_url_to_page ( $url )
{
	if (($slash_pos = strrpos( $url, '/' )) !== FALSE) $url = substr( $url, $slash_pos + 1);
	if (($quest_pos = strpos( $url, '?' )) !== FALSE) $url = substr( $url, 0, $quest_pos );
	return( $url );
}

/**
 * liefert zu einer page den action code (AC)
 * 
 * @param	$page		Page-Name
 * @return	AC			Action-Code 
 */
function get_page_to_ac ( $page )
{
	global $page_action_codes;

	$ac = AC_ENABLED;
	if (array_key_exists( $page, $page_action_codes )) $ac = $page_action_codes[$page];

	return $ac;
}

/**
 * liefert zu einer url den action code (AC)
 * 
 * @param	$url		URL
 * @return	AC			Action-Code
 */
function get_url_to_ac ( $url )
{
	return get_page_to_ac( get_url_to_page( $url ) );
}

/**
 * liefert zu einer url den kompletten html-code
 * 
 * als string wir geliefert "<a ref='$url' title='$alt_title'>$title</a>"
 * dazu wird geprueft, ob die session berechtig ist, die url aufzurufen
 * 
 * ist die session nicht berechtigt, wird der HTML-Anchor weggelassen,
 * ist zusaetzlich hide_disabled gesetzt, wird eine leerer String ausgegeben 
 * 
 * @param		$url				URL
 * @param		$title				Titel/Beschriftung des Links
 * @param		$hide_disabled		Titel wird auch angezeigt, wenn die
 * 									session nicht berechtigt ist (ja/nein)
 * @param		$alt_title			alternativer title im anchor
 * @return		string				HTML-Code
 */
function get_html_url ( $url, $title, $hide_disabled=FALSE, $alt_title="" ) 
{
	$html = "";
	$ac = get_url_to_ac( $url );
	if (core_arm::getInstance()->ac_permitted( $ac ))
	{
		// permitted
		$html = sprintf( "<a href='%s' title='%s'>%s</a>", $url, $alt_title, $title );
	}
	else
	{
		// not permitted
		if (! $hide_disabled)
		{
			$html = $title;
		}
	}
	return $html;
}

/**
 * fetch value for arg_name from POST and GET
 * 
 * ATTENTION: do not use this function directly, use http_get_<type> instead!
 * 
 * @param	arg_name		name of argument
 * @param	arg_default		default value, if not found
 * @return	string			value or null (if not found)
 */
function http_get_value ( $arg_name, $arg_default = null )
{
	if (strlen($arg_name)==0) throw new core_arg_exception( "http_get_value(): empty arg_name" );
	
	if ( isset( $_POST[$arg_name] ) )
	{
		$arg_value = $_POST[$arg_name];
	}
	else
	{
		if ( isset( $_GET[$arg_name] ) )
		{
			$arg_value = $_GET[$arg_name];
		}
		else
		{
			$arg_value = $arg_default;
		}
	}

	if (($arg_value != null) && (strlen($arg_value)==0)) $arg_value = $arg_default;

	// clean up
	if ($arg_value !== null)
	{
		$arg_value = stripslashes( $arg_value );
		$arg_value = trim( $arg_value );
	}
	
	return $arg_value;
}

/**
 * fetch array of values for arg_name from POST and GET
 * 
 * @param	arg_name	name of argument
 * @return	array		value or null (if not found)
 */
function http_get_array ( $arg_name )
{
	if (strlen($arg_name)==0) throw new core_arg_exception( "http_get_array(): empty arg_name" );
	
	if ( isset( $_POST[$arg_name] ) )
	{
		$result = $_POST[$arg_name];
	}
	else
	{
		if ( isset( $_GET[$arg_name] ) )
		{
			$result = $_GET[$arg_name];
		}
		else
		{
			return null;
		}
	}

	// clean up
	if (is_array($result))
	{
		foreach ($result as $key => $value)
		{
			$value = stripslashes( $value );
			$result[$key] = trim( $value );
		}
	}
	else
	{
		$result = stripslashes( $result );
		$result = trim( $result );
	}

	return $result;
}

// TODO: unused function, remove 
/**
 * fetch bool-value for arg_name from POST and GET
 * 
 * @param	$arg_name		name of argument
 * @param	$arg_default	default value
 * @return	bool			value of argument
 */
function http_get_bool ( $arg_name, $arg_default = null )
{
	if (($arg_value = http_get_value( $arg_name, $arg_default )) == null) return null;
	
	// type check
	if ((string)((bool)$arg_value) != $arg_value)
	{
		throw new core_arg_exception( "http_get_bool($arg_name): data type corruption" );
	}

	return (bool)$arg_value;
}

/**
 * fetch int-value for arg_name from POST and GET
 * 
 * @param	$arg_name		name of argument
 * @param	$arg_default	default value or NULL if required
 * @return	int				value of argument
 */
function http_get_int ( $arg_name, $arg_default = null )
{
	if (($arg_value = http_get_value( $arg_name, $arg_default )) == null) return null;

	// type check
	if ((string)((int)$arg_value) != $arg_value)
	{
		throw new core_arg_exception( "http_get_int($arg_name,$arg_default): data type corruption" );
	}
	
	return (int)$arg_value;
}

/**
 * fetch float-value for arg_name from POST and GET
 * 
 * @param	$arg_name		name of argument
 * @param	$arg_default	default value
 * @return	float			value of argument
 */
function http_get_float ( $arg_name, $arg_default = null )
{
	if (($arg_value = http_get_value( $arg_name, $arg_default )) == null) return null;
	
	// type check
	if ((string)((float)$arg_value) != $arg_value)
	{
		throw new core_arg_exception( "http_get_float($arg_name): data type corruption" );
	}

	return (float)$arg_value;
}

/**
 * fetch string-value for arg_name from POST and GET
 * 
 * @param	$arg_name		name of argument
 * @param	$arg_default	default value
 * @return	string			value of argument
 */
function http_get_string ( $arg_name, $arg_default = null )
{
	if (($arg_value = http_get_value( $arg_name, $arg_default )) == null) return null;

	// clean up
//	$arg_value = htmlentities( $arg_value, ENT_QUOTES, 'ISO-8859-1' );

	return (string)$arg_value;
}

/**
 * fetch password-value for arg_name from POST and GET
 * 
 * @param	$arg_name		name of argument
 * @param	$arg_default	default value
 * @return	string			value of argument
 */
function http_get_passwd ( $arg_name, $arg_default = null )
{
	if (($arg_value = http_get_value( $arg_name, $arg_default )) == null) return null;

	return (string)$arg_value;
}

// print a html option line with given and text
// mark as selected if value and ref are equal
//
function cat_option ( $value, $ref, $text )
{
	if (! is_array($ref)) $ref = array( $ref );
	
	$xx="";
	if (in_array( $value, $ref))
	{
		$xx="selected='selected'";
	}
	printf("<option value='%s' %s>%s</option>\n", $value, $xx, htmlentities( $text ));

	return;
}

// print a complete html option list with key/value pairs
// given by value_array
// ref_id marks the selected option
// arg_name is the argument name for the formular
// default is the default entry in list
//
function cat_array ( &$value_array, $ref_id, $excl_id, $arg_name, $default="", $form_name="", $default_value=0 )
{
	if (strlen($form_name)==0)
	{
		printf( "<select name='%s'>", $arg_name	);
	}
	else
	{
		printf( "<select name='%s' onchange='%s.submit()'>", $arg_name, $form_name );
	}
	if ((strlen($default)!=0) && ($default_value != $excl_id)) cat_option( $default_value, $ref_id, $default );

	foreach( $value_array as $key => $value )
	{
		if ($key != $excl_id) cat_option( $key, $ref_id, $value );
	}
	printf( "</select>"	);

	return;	
}
