<?php
# php-cmdb - a php based configuration management database

# Copyright (C) 2008  Dr. Michael Scheibe - michael@nofiles.de

# php-cmdb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# php-cmdb is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with php-cmdb.  If not, see <http://www.gnu.org/licenses/>.

/**
* @author Dr. Michael Scheibe
* @package core
* 
* $Id: core_list.inc.php 374 2009-05-21 15:12:25Z micha $
*/

require_once 'core/core_object.inc.php';

/**
 * abstract class to manage a list of core_db_objects
 */
abstract class core_db_list
{
	/**
	 * array with core_db_objects, incremental numbered from 0, ordered
	 * by DB-query from methode "get_query"
	 */
	protected $object_list	= array();
	
	/**
	 * array with the IDs of the core_db_objects as key, values are position
	 * in object_list 
	 */
	protected $index_list	= array();

	/**
	 * constructor
	 */
	protected function __construct ()
	{
		$this->load_objects();
		return;
	}

	/**
	 * reply title of object with $_idx as ID
	 * 
	 * @return	string 
	 */
	public function get_object_name ( $_idx )
	{
		$name = "*";
		if ($this->object_exists( $_idx ))
		{
			$name = $this->get( $_idx )->get_title();
		}
		return $name;
	}
	
	/**
	 * reply number of objects in the list
	 * 
	 * @return	integer
	 */
	public function count ()
	{
		return count($this->object_list);
	}
		
	/**
	 * replay an array with IDs of all objects in the list
	 * 
	 * @return	array
	 */
	public function get_idx ()
	{
		return array_keys( $this->index_list );
	}

	/**
	 * reply TRUE, if object with $_idx as ID is in the list, otherwise FALSE
	 * 
	 * @param 	integer	$_idx
	 * @return	boolean	
	 */
	public function object_exists ( $_idx )
	{
		return array_key_exists( $_idx, $this->index_list );
	}
	
	/**
	 * replay object at position $_pos,
	 * 
	 * method reply null, if _pos is not in range
	 * 
	 * @param		integer		$_pos
	 * @return		core_db_object
	 */
	public function & at ( $_pos )
	{
		// init
		$object = null;

		// check: valid position		
		if ($_pos <0 ) return $object;
		if ($_pos >= $this->count()) return $object;
					
		// deliver
		return $this->object_list[ $_pos ];
	}
	
	/**
	 * replay object with $_idx as ID from list
	 * 
	 * method reply null, if object is not in list
	 * 
	 * @param		integer		$_idx
	 * @return		core_db_object
	 */
	public function & get ( $_idx )
	{
		// init
		$object = null;

		// check: object in list		
		if (! $this->object_exists( $_idx )) return $object;
		
		$object = & $this->object_list[ $this->index_list[ $_idx ] ];
		return $object;
	}

	/**
	 * add $object at position $_pos
	 * 
	 * all other objects in the list from this position ahead
	 * will shift up one position
	 * 
	 * important: modified position data of the listed objects are not stored
	 * 
	 * @param	core_db_object	$object
	 * @param	integer			$_pos
	 * @return	none
	 */
	public function add_to_list ( & $object, $_pos )
	{
		// check: class
		if (! is_subclass_of( $object, "core_db_object" ))
		{
			trigger_error( 'not a subclass of core_db_object', E_USER_ERROR );
			exit;
		}

		// cache count
		$c		= $this->count();
		
		// check: position in range
		if ($_pos < 0) return;
		if ($_pos > $c) return;
		
		// shift higher objects up
		for ($i=$c; $i>$_pos; $i--)
		{
			$_object = & $this->object_list[ $i - 1 ];
			$this->replace( $_object, $i );
		}
		
		// replace new object in the list
		$this->replace( $object, $_pos );

		return;
	}

	/**
	 * remove object with $_idx as ID from list
	 * 
	 * important: modified variables of the listed objects are not stored
	 * 
	 * @param	integer		$_idx
	 * @return	core_db_object
	 */
	public function remove_from_list ( $_idx )
	{
		$object = null;
		
		// check: object in list		
		if (! $this->object_exists( $_idx )) return $object;

		// get the object
		$_pos = $this->index_list[ $_idx ];
		$object = & $this->object_list[ $_pos ];
		
		// shift higher objects down
		$c = $this->count();
		for ($i=$_pos+1; $i<$c; $i++)
		{
			$_object = & $this->object_list[ $i ];
			$this->replace( $_object, $i - 1 );
		}

		// unset highest position
		unset( $this->object_list[ $c - 1 ] );

		return $object;
	}

	/**
	 * swap objects in list between positions $_pos1 and $_pos2
	 * 
	 * important: modified variables of the listed objects are not stored
	 * 
	 * @param	integer		$_pos1
	 * @param	integer		$_pos2
	 * @return	none
	 */
	public function swap ( $_pos1, $_pos2 )
	{
		// check: positions in range
		if ($_pos2 == $_pos1) return;
		if (($_pos1 < 0) || ($this->count() <= $_pos1 )) return;
		if (($_pos2 < 0) || ($this->count() <= $_pos2 )) return;

		// check order
		if ($_pos2 < $_pos1) { $_pos = $_pos2; $_pos2 = $_pos1; $_pos1 = $_pos; }

		// get objects
		$_object1 = & $this->object_list[ $_pos1 ];
		$_object2 = & $this->object_list[ $_pos2 ];
		
		// replace
		$this->replace( $_object1, $_pos2 );
		$this->replace( $_object2, $_pos1 );
		
		return;
	}

	/**
	 * call a db_update for all objects in the list
	 * 
	 * @return	boolean
	 */
	public function db_update ()
	{
		$rc = TRUE;
		
		foreach( $this->object_list as $db_object )
		{
			$rc = $rc && $db_object->db_update();
		}
		
		return $rc;
	}
	
	/**
	 * print out a HTML option list with the objects from the list
	 * 
	 * @param	array		$ref_id		array of selected IDs
	 * @param	array		$excl_id	array of IDs to exclude from output
	 * @param	string		$arg_name	name for the HTML option element
	 * @param	array		$extras		array with extra elements to output
	 * @param	string		$form_name	name of HTML form element (default empty)
	 * @param	boolean		$multiple	allow multi selection (default FALSE)
	 * @return	none
	 */
	public function html_options ( $ref_id, $excl_id, $arg_name, $extras, $form_name="", $multiple=FALSE )
	{
		if (! is_array($ref_id)) trigger_error( "ref_id is not an array!", E_USER_ERROR );
		if (! is_array($excl_id)) trigger_error( "excl_id is not an array!", E_USER_ERROR );
		if (! is_array($extras)) trigger_error( "extras is not an array!", E_USER_ERROR );

		// begin HTML select element
		$onchange_str = "";
		if (strlen($form_name)>0) $onchange_str = sprintf( " onchange='%s.submit()'", $form_name);
		$multi_str = "";
		if ($multiple)
		{
			$multi_str = sprintf( " size='5' multiple='multiple'" );
			$arg_name = $arg_name . "[]";
		}

		printf( "<select name='%s'%s%s>", $arg_name, $onchange_str, $multi_str );
		
		// add extras
		if (count($extras)>0)
		{
			foreach ($extras as $key => $value)
			{
				if (! in_array( $key, $excl_id))
				{
					cat_option( $key, $ref_id, $value );
				}
			}
		}
		
		// list 
		foreach ($this->object_list as $db_object)
		{
			if (! in_array( $db_object->get_id(), $excl_id))
			{
				$this->cat_option( $db_object, $ref_id );
			}
		}

		// end HTML select element
		printf( "</select>"	);

		return;
	}

	/**
	 * print out a HTML checkbox list with the objects from the list
	 * 
	 * @param	array	$ref_array		array with IDs of all checked objects
	 * @param	integer	$excl_id		exclude object with this ID from output
	 * @param	string	$arg_name		name for the HTML checkbox element
	 * @param	string	$form_name		name for the HTML form element 
	 * @return	none
	 */
	public function html_checkbox ( $ref_array, $excl_id, $arg_name, $form_name="" )
	{
		$cra = count($ref_array);
		$submit_str="";
		if (strlen($form_name)!=0) $submit_str="onchange='$form_name.submit()'";
		
		foreach( $this->object_list as $db_object )
		{
			// mark as checked, if ref_array is empty (means all checked) or object-id
			// is in ref_array 
			$check_str="";
			if (
				($cra==0)
				||
				(in_array($db_object->get_id(), $ref_array))
				) $check_str="checked='checked'";

			printf("<input type='checkbox' name='%s[]' value='%d' %s %s> \n",
				$arg_name, $db_object->get_id(), $check_str, $submit_str
				);
			$this->cat_checkbox( $db_object );
		}

		return;
	}

	/**
	 * load objects from database into list
	 * 
	 * @return	none
	 */
	private function load_objects ()
	{ 
		$this->object_list	= array();
		$this->index_list	= array();
		$db					= core_db::getInstance();
		$id_name			= $this->get_id_name();
		$query_array		= $this->get_query();
		$j					= 0;
		foreach ($query_array as $query)
		{
//echo "DEBUG (core_list::load_objects): $query<br>\n";
			$result = $db->query( $query );
			$num    = $db->num_rows( $result );
			for ($i=0; $i<$num; $i++)
			{
				$data = $db->fetch($result);
				$data_id = $data[$id_name];
				if ($this->object_exists( $data_id )) continue;

				$this->object_list[$j] = $this->create_object( 0 );
				if (! is_subclass_of( $this->object_list[$j], "core_db_object" ))
				{
					trigger_error( 'not a subclass of core_db_object', E_USER_ERROR );
					exit;
				}

				$this->object_list[$j]->set_id( $data_id );
				$this->object_list[$j]->load_from_array( $data );
				$this->index_list[ $data_id ] = $j;
				$j++;
			}
		}
		return;
	}
	
	/**
	 * replay name of ID field
	 * 
	 * @return	string
	 */
	abstract protected function get_id_name ();
	
	/**
	 * replay array with SQL queries to select all objects for the list
	 * 
	 * IMPORTANT: the queries must deliver ALL fields, requiered to create
	 * an object with the method create_object
	 * 
	 * @return	array
	 */
	abstract protected function get_query ();
	
	/**
	 * replace an object to a position in the list
	 * 
	 * @param	core_object		$_object
	 * @param	integer			$_pos
	 */
	protected function replace ( & $_object, $_pos )
	{
		$_idx = $_object->get_id();
		$this->object_list[ $_pos ] = & $_object;
		$this->index_list[ $_idx ] = $_pos;
		$_object->set_list_pos( $_pos );
 
		return;
	}

	/**
	 * print out an HTML option element for db_object
	 * 
	 * @param	core_db_object	$db_object
	 * @param	array			$ref_id		current selected IDs
	 * @return	none
	 */
	protected function cat_option ( &$db_object, $ref_id )
	{
		cat_option( $db_object->get_id(), $ref_id, $db_object->get_title() );
	}

	/**
	 *	print out an HTML checkbox element for db_object
	 *
	 * @param	core_db_object	$db_object
	 * @return	none
	 */
	protected function cat_checkbox ( &$db_object )
	{
		echo $db_object->get_title();
	}
	
	/**
	 * create a new object for the list
	 * 
	 * normaly the new object is loaded from the database
	 * 
	 * @param	integer		$_idx
	 * @return	core_db_object
	 */
	abstract protected function create_object ( $_idx );	
}
