<?php
# php-cmdb - a php based configuration management database

# Copyright (C) 2008  Dr. Michael Scheibe - michael@nofiles.de

# php-cmdb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# php-cmdb is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with php-cmdb.  If not, see <http://www.gnu.org/licenses/>.

/**
* @author Dr. Michael Scheibe
* @package core
* 
* $Id: core_navi_list.inc.php 338 2009-04-10 12:09:15Z micha $
*/

require_once 'core/core_list.inc.php';

/**
 * abstract class to navigate by page through a list of core_db_objects
 */
abstract class core_navi_list extends core_db_list
{
	/**
	 * items per page
	 */
	protected $page_size		= 1;

	/**
	 * current page
	 */
	protected $page_num			= 0;
	
	/**
	 * last page
	 */
	protected $page_last		= 0;
	
	/**
	 * current item
	 */
	protected $item_num			= 0;
	
	/**
	 * initialize list and recalibrate
	 * paging
	 * 
	 * @return	none
	 */
	protected function __construct()
	{
		parent::__construct();
		$this->recalibrate();
//		$this->debug_info("core_navi_list::__construct");
		return;
	}

	/**
	 * for debug purposses only
	 */
	public function debug_info( $location="" )
	{
		echo "DEBUG($location): PAGE-SIZE " . $this->page_size . " / PAGE-NUM " . $this->page_num . " / PAGE-LAST " . $this->page_last. " / ITEM-NUM " . $this->item_num . " / COUNT " . $this->count() . "<br>\n";
	}


	// get methods

	/**
	 * get size of page (items per page)
	 * 
	 * @return	int							items per page
	 */
	public function get_page_size()
	{
		return $this->page_size;
	}
	
	/**
	 * get count of pages
	 * 
	 * @return	int							count of pages
	 */
	public function get_page_num()
	{
		return $this->page_num;
	}
	
	/**
	 * get number of last page
	 * 
	 * @return	int							number of last page
	 */
	public function get_page_last()
	{
		return $this->page_last;
	}
	
	/**
	 * get number of current item
	 * 
	 * @return	int							number of current item
	 */
	public function get_item_num()
	{
		return $this->item_num;
	}
	
	/**
	 * get number of first item on current page
	 * 
	 * @return	int							number of first item on current page
	 */
	public function get_page_start()
	{
		$_start_pos = ($this->page_size * $this->page_num); 
		return $_start_pos;
	}

	/**
	 * get number of last item on current page
	 * 
	 * @return	int							number of last item on current page
	 */
	public function get_page_end()
	{
		$_end_pos = ($this->page_size * ($this->page_num + 1)) - 1;
		if ($_end_pos >= $this->count()) $_end_pos = $this->count() - 1;
		return $_end_pos;
	}

	/**
	 * get count of pages
	 * 
	 * @return	int							count of pages
	 */
	public function get_page_count()
	{
		$_page_count = (int)floor( $this->count() / $this->page_size );
		return $_page_count;
	}

	// set methods
	
	/**
	 * set count of items per page
	 * 
	 * @param	int		$_size				count of items per page
	 * @return	boolean
	 */
	public function set_page_size( $_size )
	{
		if ($this->page_size == $_size) return TRUE;
		if ($_size < 1) return FALSE;
		$this->page_size = $_size;
		$this->calibrate_page_num();
		$this->calibrate_page_last();
//		$this->recalibrate();
		return TRUE;
	}	
	
	/**
	 * set number of current page
	 * 
	 * item_num is set to number of first item on page
	 * 
	 * @param	int		$_num				current page number
	 * @return	boolean
	 */
	public function set_page_num( $_num )
	{
		if ($this->page_num == $_num) return TRUE;
		if ($_num < 0) return FALSE;
		if ($_num > $this->page_last) return FALSE;
		$this->page_num = $_num;
		$this->calibrate_item_num();
		return TRUE;
	}
	
	/**
	 * set number of current item
	 * 
	 * page_num is set implicit
	 * 
	 * @param	int		$_num				number of current item
	 * @return	boolean
	 */
	public function set_item_num( $_num )
	{
		if ($this->item_num == $_num) return TRUE;
		if ($_num < 0) return FALSE;
		if ($this->count() <= $_num) return FALSE;
		$this->item_num = $_num;
		$this->calibrate_page_num();
		return TRUE;
	}
	
	/**
	 * reset all values to default
	 * 
	 * @return	none
	 */
	protected function reset()
	{
		$this->page_size = get_property("page_rows");
		$this->page_num  = 0;
		$this->page_last = 0;
		$this->item_num  = 0;

		return;
	}
	
	// actions
	
	/**
	 * goto previous page
	 * 
	 * @return	none
	 */
	public function page_prev()
	{
		if ($this->page_num > 0) $this->set_page_num( $this->page_num - 1 );
		return;
	}
	
	/**
	 * goto next page
	 * 
	 * @return	none
	 */
	public function page_next()
	{
		if ($this->page_num < $this->page_last) $this->set_page_num( $this->page_num + 1 );
		return;
	}
	
	/**
	 * goto previous item
	 * 
	 * @return	none
	 */
	public function item_prev()
	{
		if ($this->item_num > 0) $this->set_item_num( $this->item_num - 1 );
		return;
	}
	
	/**
	 * goto next item
	 * 
	 * @return	none
	 */
	public function item_next()
	{
		if ($this->item_num < ($this->count() - 1)) $this->set_item_num( $this->item_num + 1 );
		return;
	}
	
	// display
	
	/**
	 * show all items on current page
	 * 
	 * @return	none
	 */
	public function show_page()
	{
		$ps = $this->get_page_start();
		$pe = $this->get_page_end();
		for ($i=$ps; $i<=$pe; $i++)
		{
			$this->show_item( $i );
		}
		return;
	}
	
	/**
	 * display given item
	 * 
	 * Note: must be implemented
	 * 
	 * @param	int		$i					number of item to show
	 * @return	none
	 */
	abstract protected function show_item( $i ); 


	// helper

	/**
	 * check and recalculate values
	 * 
	 * @return	none
	 */
	protected function recalibrate()
	{
		// check current
		if ($this->item_num >= $this->count()) $this->item_num = $this->count() - 1;
		if ($this->item_num < 0) $this->item_num = 0;

		// check page last
		$this->calibrate_page_last();
		
		// check page num
		$this->calibrate_page_num();
		
		return;
	}

	/**
	 * recalculate and set number of current item
	 * 
	 * @return	none
	 */
	private function calibrate_item_num()
	{
		$_item_num = $this->get_page_start();
		$this->item_num = $_item_num;
		return;
	}


	/**
	 * recalculate and set number of current page
	 * 
	 * @return	none
	 */
	private function calibrate_page_num()
	{
		$_page_num = (int)floor( $this->item_num / $this->page_size );
		$this->page_num = $_page_num;
		return;
	}
	
	/**
	 * recalculate and set number of last page
	 * 
	 * @return	none
	 */
	private function calibrate_page_last()
	{
		$_page_last = (int)floor( ($this->count() - 1) / $this->page_size );
		$this->page_last = $_page_last;
//echo "DEBUG: $_page_last = " . $this->count() . " / $this->page_size <br>\n";
		return;
	}
	
}
