<?php
# php-cmdb - a php based configuration management database

# Copyright (C) 2008  Dr. Michael Scheibe - michael@nofiles.de

# php-cmdb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# php-cmdb is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with php-cmdb.  If not, see <http://www.gnu.org/licenses/>.

/**
* @author Dr. Michael Scheibe
* @package core
* 
* $Id: core_object.inc.php 338 2009-04-10 12:09:15Z micha $
*/

require_once 'core/core_db.inc.php';

/**
 * abstract base class to manage objects
 */
abstract class core_object
{
	/**
	 * constructor
	 * 
	 * @param	integer		$_id	ID des objektes
	 * @return	none
	 */
	public function __construct ( $_id = 0 )
	{
		$this->set_id( $_id );
		return;
	}
	
	/**
	 * clone object
	 * 
	 * @return	none 
	 */
	public function __clone ()
	{
		$id_name = $this->get_id_name();
		$this->$id_name = 0;
		return;
	}

	/**
	 * reply the title of the object
	 * 
	 * this method is used to present the object in lists,
	 * default is "<class_name>-<id>" 
	 * 
	 * @return	string
	 */
	public function get_title ()
	{
		$title = sprintf( "%s-%d", get_class($this), $this->get_id() );
		return $title;
	}

	/**
	 * reply the position of the object in an ordered list
	 * 
	 * default is 0
	 * 
	 * @return integer
	 */
	public function get_list_pos ()
	{
		return 0;
	}

	/**
	 * set position of the object in an ordered list
	 * 
	 * @param	integer	$_pos
	 * @return	none
	 */
	public function set_list_pos ( $_pos )
	{
		return;
	}

	// load methods
	
	/**
	 * load all object fields from an array
	 *
	 * the array delivers as key-value pairs the content
	 * for the object fields, its important that one key is the
	 * name given by the method "get_id_name" and the related
	 * valued represents the ID of the object, only if
	 * this condition is matched, all other keys
	 * and values from the array will be loaded into the object
	 *  
	 * @param	array		$_array		key-value pairs
	 * @return	boolean
	 */
	public function load_from_array ( $_array )
	{
		// check: type of argument
		if (! is_array( $_array ))
		{
			trigger_error( 'argument is not an array', E_USER_ERROR );
			exit;
		}
		
		// check: id_name is in array		
		$id_name = $this->get_id_name();
		if (! array_key_exists( $id_name, $_array))
		{
			trigger_error( 'id_name not found in array', E_USER_ERROR );
			exit;
		}
		
		// check: correct ID
		if ($_array[ $id_name ] != $this->get_id()) return FALSE;

		// load
		foreach ($this as $key => $value)
		{
			if (($key != $id_name) && (isset($_array[$key]))) $this->$key = $_array[$key];
		}
		
		return TRUE;
	}
	
	// set methods
	
	/**
	 * set ID of the object
	 * 
	 * @param		integer		$_id
	 * @return		boolean
	 */
	abstract public function set_id ( $_id ); 
	
	// get methods

	/**
	 * replay name of the ID field
	 * 
	 * @return	string
	 */
	abstract protected function get_id_name ();

	/**
	 * replay ID of the object
	 * 
	 * @return	integer
	 */
	abstract public function get_id ();	
	
}

/**
 * abstract class to manage objects mapped to a database
 */
abstract class core_db_object extends core_object
{
	/**
	 * count modifications in field values since last load or update
	 */
	private $modify_count	= 0;

	/**
	 * constructor
	 */
	public function __construct ( $_id = 0 )
	{
		parent::__construct ( $_id );
		$this->db_load();
		return;
	}


	/**
	 * replay name of database table
	 * 
	 * @return	string
	 */
	abstract protected function get_table_name ();


	// modify handling

	/**
	 * mark as modified excplicit
	 * 
	 * @return	none
	 */
	public function activate_modify ()
	{
		$this->mark_modified();
	}
	
	/**
	 * reply TRUE, if object is modified
	 * 
	 * reply TRUE, if modify_count is not 0, otherwise FALSE
	 * 
	 * @return		boolean		TRUE, wenn modify_count != 0
	 */
	public function is_modified ()
	{
		return ($this->modify_count != 0);
	}
	
	/**
	 * mark object as modified
	 * 
	 * implicit increment the modify_count
	 * 
	 * @return	none
	 */
	protected function mark_modified ()
	{
		$this->modify_count++;
	}
	
	/**
	 * mark as un-modified excplicit
	 * 
	 * set modify_count to 0
	 * 
	 * @return	none
	 */
	protected function mark_unmodified ()
	{
		$this->modify_count = 0;
	}

	// valididerung
	
	/**
	 * reply TRUE, if all fields are valid
	 * 
	 * @return		boolean
	 */
	abstract public function is_valid();

	// db actions high level

	/**
	 * load object fields from database
	 * 
	 * ID of object must not be 0 (it means, th eobject must be stored in
	 * database already)
	 * 
	 * @return		boolean
	 */	
	public function db_load ()
	{
		if ($this->get_id() == 0) return FALSE;
		$rc = $this->db_load_impl();
		if ($rc) $this->mark_unmodified();
		return $rc;
	}

	/**
	 * insert object into database
	 * 
	 * ID of object must be 0 (it means, the object is not already stored), the
	 * validation check must reply TRUE
	 * 
	 * @return		boolean
	 */
	public function db_insert ()
	{
		if ($this->get_id() != 0) return FALSE;
		if (! $this->is_valid()) return FALSE;
		$rc = $this->db_insert_impl();
		if ($rc) $this->mark_unmodified();
		return $rc;
	}
	
	/**
	 * updates object in database
	 * 
	 * stores current object values into the database, the ID of
	 * the object must not be 0, the validation check must reply TRUE
	 * 
	 * @return		boolean
	 */
	public function db_update ()
	{
		if (! $this->is_modified()) return TRUE;
		if ($this->get_id() == 0) return FALSE;
		if (! $this->is_valid()) return FALSE;
		$rc = $this->db_update_impl();
		if ($rc) $this->mark_unmodified();
		return $rc;
	}
	
	/**
	 * delete object from database
	 * 
	 * ID of the object must not be 0, after removed from the database
	 * ID will set to 0 
	 * 
	 * @return		boolean
	 */
	public function db_delete ()
	{
		if ($this->get_id() == 0) return FALSE;
		$rc = $this->db_delete_impl();
		if ($rc) $this->set_id(0);
		return $rc;
	}

	// db actions low level
	
	/**
	 * implementation method for database access
	 * 
	 * @return		boolean
	 */
	protected function db_load_impl ()
	{
		$id_name	= $this->get_id_name();
		$tb_name	= $this->get_table_name();
		$db			= core_db::getInstance();
		$query		= "SELECT * FROM $tb_name WHERE $id_name='" . $this->$id_name . "'";
		$result = $db->query( $query );
		if ($db->num_rows($result)!=1)
		{
			throw new core_db_exception( "unexpected number of rows" );
		}

		$data= $db->fetch( $result );
		return $this->load_from_array( $data );
	}

	/**
	 * implementation method for database access
	 * 
	 * @return		boolean
	 */
	protected function db_insert_impl ()
	{
		$id_name	= $this->get_id_name();
		$tb_name	= $this->get_table_name();
		$db			= core_db::getInstance();
		$q_attr		= "";
		$q_val		= "";
		foreach( $this as $key => $value )
		{
			if ($key == "modify_count") continue;
			if ($key == $id_name) continue;
			if (strlen($q_attr)!=0)
			{
				$q_attr = $q_attr . ",";
				$q_val  = $q_val  . ",";
			}
			$value_escaped = $db->escape_string( $value );
			$q_attr = $q_attr . $key;
			$q_val  = $q_val  . "'" . $value_escaped . "'";
		}
		$query		= "INSERT INTO $tb_name (".
			$q_attr.") VALUES (".
			$q_val.")";
		$result = $db->query( $query );

		// id holen
		$this->$id_name = $db->id();

		return TRUE;
	}

	/**
	 * implementation method for database access
	 * 
	 * @return		boolean
	 */
	protected function db_update_impl ()
	{
		$id_name	= $this->get_id_name();
		$tb_name	= $this->get_table_name();
		$db			= core_db::getInstance();
		$q_attr		= "";
		foreach( $this as $key => $value )
		{
			if ($key == "modify_count") continue;
			if ($key == $id_name) continue;
			if (strlen($q_attr)!=0)
			{
				$q_attr = $q_attr . ",";
			}
			$value_escaped = $db->escape_string( $value );
			$q_attr = $q_attr . "$key='$value_escaped'";
		}
		$query	= "UPDATE $tb_name SET $q_attr WHERE $id_name='" . $this->$id_name . "' LIMIT 1";
		$result = $db->query( $query );

		return TRUE;
	}

	/**
	 * implementation method for database access
	 * 
	 * @return		boolean
	 */
	protected function db_delete_impl ()
	{
		$id_name	= $this->get_id_name();
		$tb_name	= $this->get_table_name();
		$db			= core_db::getInstance();
		$query		= "DELETE FROM $tb_name " .
			"WHERE $id_name='" . $this->$id_name . "' LIMIT 1";
		$result = $db->query( $query );

		return TRUE;
	}
	
}
