<?php
# php-cmdb - a php based configuration management database

# Copyright (C) 2008  Dr. Michael Scheibe - michael@nofiles.de

# php-cmdb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# php-cmdb is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with php-cmdb.  If not, see <http://www.gnu.org/licenses/>.

/**
* @author Dr. Michael Scheibe
* @package core
* 
* $Id: core_processor.inc.php 365 2009-04-14 18:48:20Z micha $
*/


/**
 * abstrakte klasse fuer das processing
 * 
 * muss messages aufsammeln koennen, muss den action status
 * verwalten und ausliefern
 * 
 */
abstract class core_processor
{
	/**
	 * array mit den meldungen und deren status
	 * [][0] = status
	 * [][1] = meldung
	 */
	protected $messages	= array();
	
	/**
	 * gesamt-status, groesster bisher eingetragener wert
	 */
	protected $status = STATUS_OK;
	
	/**
	 * constructor
	 */
	public function __construct ()
	{
		return;
	}

	/**
	 * liefert die anzahl der meldungen
	 */
	public function get_message_count ()
	{
		return count($this->messages);
	}

	/**
	 * liefert den gesamt-status
	 */
	public function get_status ()
	{
		return $this->status;
	}

	/**
	 * liefert das array fuer den eintrag $_idx
	 * [0] = status
	 * [1] = meldung
	 */
	public function get_message ( $_idx )
	{
		return $this->messages[ $_idx ];
	}

	/**
	 * liefert den letzten meldungstext aus
	 */
	public function get_last_message ()
	{
		$last_msg = get_string('NoMessageAvailable_dsc');
		if ($this->get_message_count() > 0) $last_msg = $this->messages[ $this->get_message_count() -1 ][ 1 ]; 
		return $last_msg;
	}

	/**
	 * fuegt eine meldung inkl. status ein
	 */
	public function add_message ( $_message, $_status = STATUS_ERROR )
	{
		if (strlen($_message)==0) return;
		
		$idx = count($this->messages);
		$this->messages[$idx] = array( $_status, $_message );

		if ($_status > $this->status) $this->status = $_status;
		return;
	}

	/**
	 * gibt das komplette message-array per print_r aus
	 * 
	 * dient vor allem zu debug-zwecken, sollte im normalen
	 * verlauf nicht genutzt werden
	 */
	public function print_messages ()
	{
		print_r($this->messages );
		return;
	}

	public function html_messages ( $final_message='', $rc_url=STATUS_OK )
	{
		printf( "<div id='status'>\n" );
		foreach ($this->messages as $message )
		{
			html_status( $message[0], $message[1] );
		}

		if (strlen($final_message)!=0)
		{
			html_status( $this->get_status(), $final_message, $rc_url );
		}
		printf( "</div>\n" );		
	}

	public function html_status ( $rc_url )
	{
		printf( "<div id='status'>\n" );
		html_status( $this->get_status(), $this->get_last_message(), $rc_url );
		printf( "</div>\n" );		
	}

	/**
	 * loescht alle meldungen und setzt den status auf STATUS_OK zurueck
	 */
	public function reset_messages ()
	{
		$this->messages = array();
		$this->status = STATUS_OK;
	}


}
 
/**
 * abstract class to manage db_table objects
 */
abstract class core_db_processor extends core_processor
{
	protected $db_object	= null;
	
	/**
	 * create a new instance
	 * 
	 * use the given ID, to load the object content from the 
	 * database, if $_id is 0 and $_arg_name is not empty, try
	 * to load ID from HTTP arg with the $_arg_name
	 * 
	 * @param	integer		$_id		ID of the db_table object
	 * @param	string		$_arg_name	name of the HTTP-arg
	 * @return	none
	 */	
	public function __construct ( $_id=0, $_arg_name="" )
	{
		parent::__construct();
		if ( ($_id == 0) && (strlen($_arg_name)>0) ) $_id = http_get_int( $_arg_name, $_id );
		$this->create_db_object( $_id );
		return;
	} 

	public function __clone ()
	{
		$this->db_object = clone $this->db_object;
	}

	/**
	 * overload object content from HTTP-args
	 * 
	 * @return	none
	 */	
	abstract public function load_from_args ();

	/**
	 * overload object content from database
	 */
	public function refresh()
	{
		try
		{
			$this->db_object->db_load();
		}
		catch (core_db_exception $e)
		{
			$message = sprintf( "%s ('%s')", get_string('DBError_dsc'), $e->get_message() );
			$this->add_message( $message );
		}
		catch (core_data_exception $e)
		{
			$message = sprintf( "%s ('%s')", get_string('ValidationError_dsc'), $e->get_message() );
			$this->add_message( $message );
		}
		return;
	}

	/**
	 * reply ID of the object
	 * 
	 * @return	integer	
	 */
	public function get_id ()
	{
		return $this->db_object->get_id();
	}

	// action methoden

	/**
	 * insert object into database
	 *
	 * @return	boolean		status
	 */
	public function action_insert ()
	{
		$rc = FALSE;
		try {
			// args laden
			$this->load_from_args();		
	
			// valid check
			$this->valid_for_insert();

			// pre
			$this->insert_pre();

			// speichern
			$this->db_object->db_insert();
			
			// post
			$this->insert_post();

			// message
			$this->add_message( get_string('InsertSuccessful_dsc'), STATUS_OK );
			$rc = TRUE;
		}
		catch (core_db_exception $e)
		{
			$message = sprintf( "%s ('%s')", get_string('DBError_dsc'), $e->get_message() );
			$this->add_message( $message );
		}
		catch (core_data_exception $e)
		{
			$message = sprintf( "%s ('%s')", get_string('ValidationError_dsc'), $e->get_message() );
			$this->add_message( $message );
		}
		return $rc;
	}

	/**
	 * update database from object content
	 * 
	 * @return	boolean		status 
	 */
	public function action_update ()
	{
		$rc = FALSE;
		try {
			// args laden
			$this->load_from_args();		
			
			// valid check
			$this->valid_for_update();
	
			// pre
			$this->update_pre();

			// speichern
			$this->db_object->db_update();
			
			// post
			$this->update_post();

			// message
			$this->add_message( get_string('UpdateSuccessful_dsc'), STATUS_OK );
			$rc = TRUE;
		}
		catch (core_db_exception $e)
		{
			$message = sprintf( "%s ('%s')", get_string('DBError_dsc'), $e->get_message() );
			$this->add_message( $message );
		}
		catch (core_data_exception $e)
		{
			$message = sprintf( "%s ('%s')", get_string('ValidationError_dsc'), $e->get_message() );
			$this->add_message( $message );
		}
		return $rc;
	}

	/**
	 * delete object content from database
	 * 
	 * @return	boolean		status 
	 */
	public function action_delete ()
	{
		$rc = FALSE;
		try
		{
			// valid check
			$this->valid_for_delete();
	
			// pre
			$this->delete_pre();

			// loeschen
			$this->db_object->db_delete();
			
			// post
			$this->delete_post();

			// message
			$this->add_message( get_string('DeleteSuccessful_dsc'), STATUS_OK );
			$rc = TRUE;
		}
		catch (core_db_exception $e)
		{
			$message = sprintf( "%s ('%s')", get_string('DBError_dsc'), $e->get_message() );
			$this->add_message( $message );
		}
		catch (core_data_exception $e)
		{
			$message = sprintf( "%s ('%s')", get_string('ValidationError_dsc'), $e->get_message() );
			$this->add_message( $message );
		}
		return $rc;
	}

	// pre- / post- actions

	/**
	 * pre insert actions
	 * 
	 * place sepcific actions to run before insert here
	 * 
	 * @return	none
	 */	
	protected function insert_pre ()
	{
		return;
	}

	/**
	 * post insert actions
	 * 
	 * place sepcific actions to run after insert here
	 * 
	 * @return	none
	 */	
	protected function insert_post ()
	{
		return;
	}

	/**
	 * pre update actions
	 * 
	 * place sepcific actions to run before update here
	 * 
	 * @return	none
	 */	
	protected function update_pre ()
	{
		return;
	}

	/**
	 * post update actions
	 * 
	 * place sepcific actions to run after update here
	 * 
	 * @return	none
	 */	
	protected function update_post ()
	{
		return;
	}

	/**
	 * pre delete actions
	 * 
	 * place sepcific actions to run before delete here
	 * 
	 * @return	none
	 */	
	protected function delete_pre ()
	{
		return;
	}

	/**
	 * post delete actions
	 * 
	 * place sepcific actions to run afetr delete here
	 * 
	 * @return	none
	 */	
	protected function delete_post ()
	{
		return;
	}

	// action validation
	
	/**
	 * check validation for insert
	 * 
	 * reply TRUE, if all requirements for insert are met, otherwise
	 * throw an exception
	 * 
	 * @return	bool 
	 */
	public function valid_for_insert()
	{
		return TRUE;
	}

	/**
	 * check validation for update
	 * 
	 * reply TRUE, if all requirements for update are met, otherwise
	 * throw an exception
	 * 
	 * @return	bool 
	 */
	public function valid_for_update ()
	{
		return TRUE;
	}

	/**
	 * check validation for delete
	 * 
	 * reply TRUE, if all requirements for delete are met, otherwise
	 * throw an exception
	 * 
	 * @return	bool 
	 */
	public function valid_for_delete ()
	{
		return TRUE;
	}

	/**
	 * create a new db_table object ot the related class with the
	 * given $_id
	 * 
	 * @param	integer			$_id
	 * @return	core_db_object
	 */	
	abstract protected function create_db_object ( $_id ); 

}
