<?php
# php-cmdb - a php based configuration management database

# Copyright (C) 2008  Dr. Michael Scheibe - michael@nofiles.de

# php-cmdb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# php-cmdb is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with php-cmdb.  If not, see <http://www.gnu.org/licenses/>.

/**
 * klasse zur verfolgung der CI und der Relationen
 * 
 * kann ein beziehungsnetz durchlaufen, die tiefe der
 * relationen kann begrenzt werden
 * 
 * ueber filter koennenkann das tracking auf ausgewaehlte
 * CI-Types und Relationen-Types beschraenkt werden
 * 
 */
class ci_tracker
{
	protected $base_ci_id	= null;
	protected $max_depth	= 0;
	
	/**
	 * liste der CI IDs, die bereits beim tracking
	 * erreicht wurden
	 */
	protected $ci_id_list	= array();

	/**
	 * liste der Relations IDs, die bereits beim tracking
	 * erreicht wurden
	 */
	protected $rel_id_list	= array();

	/**
	 * liste der CI-Types, die beim tracking
	 * beruecksichtigt werden sollen
	 * 
	 * ist das array leer, werden alle CI-Types
	 * beruecksichtigt
	 */
	protected $ci_type_filter	= array();

	/**
	 * liste der Relation-Types, die beim tracking
	 * beruecksichtigt werden sollen
	 * 
	 * ist das array leer, werden alle Relation-Types
	 * beruecksichtigt
	 */
	protected $rel_type_filter	= array();


	/**
	 * constructor
	 * 
	 * @param	$_ci_id			ID des Basis CI
	 * @param	$_max_depth		Maximale Tiefe der Relationen
	 */	
	public function __construct ( $_ci_id, $_max_depth )
	{
		$this->set_base_ci_id( $_ci_id );
		$this->set_max_depth( $_max_depth );
	}

	// set methoden

	/**
	 * setzt die ID des basis CI
	 */
	public function set_base_ci_id ( $_ci_id )
	{
		$db	= core_db::getInstance();
		$query = "SELECT COUNT(*) FROM ci " .
			"WHERE ci_id='$_ci_id' ";
		$result = $db->query( $query );
		if ($db->result($result,0)==1) $this->base_ci_id = $_ci_id;
	}

	/**
	 * setzt die maximale tracking tiefe
	 */
	public function set_max_depth ( $_max_depth )
	{
		if ($_max_depth < 0) return;
		$this->max_depth = $_max_depth;
	}

	/**
	 * fuegt CI-Types zur Filter-Liste hinzu
	 * 
	 * beim Tracking werden nur CI beruecksichtigt, deren
	 * CI-Type in der filter-liste enthalten sind
	 * 
	 * (falls die filter-liste leer ist, wirden alle CI beruecksichtigt)
	 * 
	 * das uebergebene filter_array muss die ID der CI-Types als Values
	 * enthalten, diese werden dann nd filter-liste uebernommen
	 * (dort als Keys !) 
	 * 
	 */
	public function add_ci_type_filter ( $filter_array )
	{
		if (! is_array($filter_array)) return;
		foreach ($filter_array as $key => $value)
		{
			$this->ci_type_filter[$value]=0;
		} 
		return;
	}
	
	/**
	 * fuegt Relation-Types zur Filter-Liste hinzu
	 * 
	 * beim Tracking werden nur Relationen beruecksichtigt, deren
	 * Relationen-Type in der filter-liste enthalten sind
	 * 
	 * (falls die filter-liste leer ist, werden alle Relationen beruecksichtigt)
	 * 
	 * das uebergebene filter_array muss die ID der Relatione-Types als Values
	 * enthalten, diese werden dann nd filter-liste uebernommen
	 * (dort als Keys !) 
	 * 
	 */
	public function add_rel_type_filter ( $filter_array )
	{
		if (! is_array($filter_array)) return;
		foreach ($filter_array as $key => $value)
		{
			$this->rel_type_filter[$value]=0;
		} 
		return;
	}
	
	/**
	 * loescht die CI-Type filter-liste
	 */
	public function reset_ci_type_filter ()
	{
		$this->ci_type_filter = array();
		return;
	}

	/**
	 * loescht die Relationen-Type filter-liste
	 */
	public function reset_rel_type_filter ()
	{
		$this->rel_type_filter = array();
		return;
	}

	// get methoden

	public function get_ci_count ()
	{
		return count($this->ci_id_list());
	}

	public function get_relation_count ()
	{
		return count($this->rel_id_list());
	}

	
	// tracking

	/**
	 * fuehrt das tracking aus
	 */
	public function run ()
	{
		$this->track_ci( $this->base_ci_id, 0 );
//		print_r( $this->ci_id_list );
	}

	/**
	 * fuehrt das tracking aus, folgt jedoch nur der
	 * angegebenen relation
	 */
	public function run_relation ( $_r_id )
	{
		$this->track_ci( $this->base_ci_id, 0, $_r_id );		
	}

	/**
	 * fuehrt das tracking fuer ein CI aus
	 * 
	 * ACHTUNG: die methode arbeitet rekursiv
	 * 
	 * @param	$_ci_id			ID des zu trackenden CI
	 * @param	$_depth			Tiefe des CI
	 * @param	$_follow_r_id	ID der zu verfolgenden Relation,
	 * 							falls 0 werden alle verfolgt
	 */
	protected function track_ci ( $_ci_id, $_depth, $_follow_r_id=0 )
	{
//		echo "DEBUG: ci_tracker::track_ci( $_ci_id, $_depth, $_follow_r_id )<br>\n";

		// handle CI
		if ($_follow_r_id==0)
		{
			// process CI only once
			if (! array_key_exists( $_ci_id, $this->ci_id_list ))
			{
				if (! $this->process_ci( $_ci_id ))
				{
					$this->ci_id_list[$_ci_id] = $_depth;
					return;
				}
			}
			
			// store reached depth
			$this->ci_id_list[$_ci_id] = $_depth;
		}

		// stop, if max_depth reached
		if ($_depth >= $this->max_depth) return;
		
		// handle childs
		$peer_array = $this->get_peers( $_ci_id );
		foreach ($peer_array as $peer)
		{
			// follow check
			if (($_follow_r_id!=0) && ($_follow_r_id!=$peer->r_id)) continue;
			
			// filter check: relation type
			if (
				(count($this->rel_type_filter)!=0)
				&&
				(! array_key_exists( $peer->r_rt_id, $this->rel_type_filter))
				)
			{
				continue;
			}
			
			// filter check: ci type
			if (
				(count($this->ci_type_filter)!=0)
				&&
				(! array_key_exists( $peer->ci_cit_id, $this->ci_type_filter))
				)
			{
				continue;
			}
			
			// follow relation
			if (! array_key_exists( $peer->r_id, $this->rel_id_list ))
			{
				// process relation
				$this->rel_id_list[$peer->r_id] = $this->process_relation( $peer->r_id, $_ci_id );

				// track to peer CI
				$this->track_ci ( $peer->ci_id, $_depth + 1);
			}

		}
	}

	/**
	 * liefert array mit allen peers des CI
	 * 
	 * folgende informationen sind als object im array
	 * - ci_id
	 * - ci_cit_id
	 * - r_id
	 * - r_rt_id
	 * 
	 * 
	 */
	protected function get_peers ( $_ci_id )
	{
		$db	= core_db::getInstance();
		$peer_array = array();

		// from relations
		$query = "SELECT r_id, r_rt_id, ci_id, ci_cit_id FROM relation, ci " .
			"WHERE r_ci1_id='$_ci_id' " .
			"AND r_ci2_id=ci_id " .
			"ORDER BY r_rt_id, ci_title";
		$result = $db->query( $query );
		$num	= $db->num_rows( $result );
		for ($i=0; $i<$num; $i++)
		{
			$data = $db->fetch_object( $result, $i );
			$peer_array[] = $data;
//			printf( "DEBUG: ci_tracker::get_peers( %d ): %d<br>\n",
//				$data->r_id, $data->ci_id );
		}		

		// to relations
		$query = "SELECT r_id, r_rt_id, ci_id, ci_cit_id FROM relation, ci " .
			"WHERE r_ci2_id='$_ci_id' " .
			"AND r_ci1_id=ci_id " .
			"ORDER BY r_rt_id, ci_title";
		$result = $db->query( $query );
		$num	= $db->num_rows( $result );
		for ($i=0; $i<$num; $i++)
		{
			$data = $db->fetch_object( $result, $i );
			$peer_array[] = $data;
//			printf( "DEBUG: ci_tracker::get_peers( %d ): %d<br>\n",
//				$data->r_id, $data->ci_id );
		}		

		return $peer_array;		
	}
	
	// processing
	
	/**
	 * verarbeitet das ci zur gelieferten ID
	 * 
	 * die methode wird beim tracking fuer jedes CI
	 * aufgerufen
	 * 
	 * die methode liefert TRUE, wenn das tracking
	 * fortgefuhert werden soll
	 * 
	 * andernfalls FALSE
	 */
	protected function process_ci ( $_ci_id )
	{
//		echo "DEBUG: ci_tracker::process_ci( $_ci_id )<br>\n";
		return TRUE;
	}
	
	/**
	 * verarbeitet die relation zur gelieferten ID
	 * 
	 * die methode wird beim tracking fuer jede
	 * relation aufgerufen
	 * 
	 * die methode liefert TRUE, wenn das tracking
	 * fortgefuhert werden soll
	 * 
	 * andernfalls FALSE
	 */
	protected function process_relation ( $_rel_id, $_ci_id )
	{
//		echo "DEBUG: ci_tracker::process_relation( $_rel_id )<br>\n";
		return TRUE;
	}
	
}
