<?php
# php-cmdb - a php based configuration management database

# Copyright (C) 2008  Dr. Michael Scheibe - michael@nofiles.de

# php-cmdb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# php-cmdb is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with php-cmdb.  If not, see <http://www.gnu.org/licenses/>.

/**
* @author		Dr. Michael Scheibe
* @package		cmdb
* @subpackage	processing
* @version		$Id: icon_list.inc.php 399 2009-07-04 07:44:28Z micha $
*/


require_once 'libs/class.ico.php';

/**
 * class for manageing a list of icons
 * 
 * implemented as singleton
 */
class icon_list
{
	static private $instance	= null;		// singleton reference
	protected $icon_dir			= "";		// (relative) directorypath to the icons
	protected $icon_full_dir	= "";		// full path to the icons
	protected $icon_sx			= 0;		// default icon width
	protected $icon_sy			= 0;		// default icon height
	private   $icon_mtime		= 0;		// last modification time of icons
	private   $preview_mtime	= 0;		// last modification time of preview
	protected $preview_png		= "";		// name of preview png file
	protected $preview_map		= "";		// name of preview map file (serialized binary)
	private   $icon_map			= array();	// map array with coords
	protected $icons			= array();	// list

	/**
	 * fabric
	 */
	static public function getInstance()
	{
		if (! self::$instance) self::$instance = new icon_list();
		return self::$instance;
	}
	
	/**
	 * construction
	 * 
	 * init fields from configuration and load icon list from directory
	 */
	protected function __construct ()
	{
		$this->icon_dir = get_property( "icon_dir" );
		$this->icon_sx  = get_property( "default_icon_size" );
		$this->icon_sy  = get_property( "default_icon_size" );
		$this->create_names();
		$this->load();
	}

	/**
	 * count of icons in the list
	 * 
	 * @return		int
	 */
	public function count()
	{
		return count($this->icons);
	}

	/**
	 * get full path to the icons
	 * 
	 * @return		string
	 */
	public function get_full_dir ()
	{
		return $this->icon_full_dir;
	}

	
	/**
	 * get name of icon with id from list
	 * 
	 * @param	$id		ID of icon
	 * 
	 * @return	string	name of icon
	 */
	public function get_name ( $id )
	{
		if (! array_key_exists( $id, $this->icons)) return "";
		return $this->icons[ $id ];
	}
	
	/**
	 * get file name of icon id from list
	 * 
	 * @param	$id		ID of icon
	 * 
	 * @return	string	file-name of icon (full path and name)
	 */
	public function get_file_name ( $id )
	{
		if (! array_key_exists( $id, $this->icons)) return "";
		$file_name = sprintf( "%s/%s", $this->icon_dir, $this->get_name( $id ) );
		return $file_name;
	}

	/**
	 * get name of cache file for the given icon name and size
	 * 
	 * @param	$name	name of icon
	 * @param	$size	width / heigth of icon
	 * 
	 * @return	string	name of icon file in cache (full path and name)
	 */
	protected function get_cache_file_name ( $name, $size )
	{
		$cache_file_name = sprintf( "%s/%s%s_%04d.png",
			get_property("cache_dir"), 
			get_property("cache_ico_prefix"), 
			$name, $size );
		return $cache_file_name;
	}

	/**
	 * get id for name from list 
	 * 
	 * @param	$name	name of icon
	 * 
	 * @return	int		ID of icon
	 */
	public function get_id ( $name )
	{
		foreach ($this->icons as $key => $value)
		{
			if (strcmp($value, $name)==0) return $key;
		}
		return false;
	}

	/**
	 * deliver icon as image resource with the requested size
	 * 
	 * @param		$name		name of icon
	 * @param		$size		requested width / height of icon
	 * 
	 * @return		resource	image
	 */
	public function get_icon_image ( $name, $size )
	{
		$icon_num = 0;
		$icon_num_max = -1;
		$icon_size_max = 0;
		
		// make shure, we have an availabe icon
		if (! in_array( $name, $this->icons )) $name = get_property( "default_icon_name" );
		if (! in_array( $name, $this->icons ))
		{
			$name = "";
			if ($this->count() > 0) $name = $this->icons[0];
		}

		// no name: return
		if (strlen($name)==0) return null;
		
		// no size: return
		if ($size <= 0) return null;

		// cache lookup
		$cache_name = $this->get_cache_file_name( $name, $size );
		if (! file_exists( $cache_name ))
		{
			if (in_array( $name, $this->icons )) 
			{
				// icon in list
				$icon = new ico( $this->icon_full_dir . "/" . $name );
				$icon_count = $icon->TotalIcons();
				
				// look for the best size match
				for ($i=0; $i<$icon_count; $i++)
				{
					$format = $icon->GetIconInfo( $i );
					if (($icon_size = $format['Width']) > $icon_size_max)
					{
						$icon_size_max = $icon_size;
						$icon_num_max = $i;
					}
					if ($icon_size==$size) $icon_num = $i;
				}
			}
			
			if ($icon_num != 0)
			{
				// load icon into image resource
				$image = $icon->GetIcon($icon_num);
				$trans_color = imagecolorallocatealpha($image,255,255,255,0);
				imagecolortransparent($image,$trans_color);
			}
			else
			{
				// icon not on list, create dummy resource
				$image = imagecreatetruecolor($size, $size);
	
				if ($icon_num_max >= 0)
				{
					$image_temp = $icon->GetIcon($icon_num_max);
					imagecopyresampled( $image, $image_temp, 0, 0, 0, 0, $size, $size, $icon_size_max, $icon_size_max);
				}
				$trans_color = imagecolorallocatealpha($image,255,255,255,0);
				imagecolortransparent($image,$trans_color);
			}

			// store in cache file
			imagepng( $image, $cache_name );
		}
		else
		{
			// load from cache file 
			$image = imagecreatefrompng( $cache_name );			
		}
		
		return $image;
	}
	
	/**
	 * deliver preview image
	 */
	public function get_preview_image ()
	{
		header ("Content-type: image/png");
		readfile( $this->preview_png );
		
		return;
	}

	/**
	 * get a HTML string with an img-tag referencing the icon name and given size
	 * 
	 * @param	$name		name of icon
	 * @param	$sx			width of icon
	 * @param	$sy			height of icon
	 * @param	$use_dummy	TRUE: use default icon, if requested icon does not exists
	 * 
	 * @return	string		HTML string with img
	 */
	public function get_html_imx ( $name, $sx=0, $sy=0, $use_dummy=FALSE )
	{
		$url = "";

		// check: name
		if ($this->get_id( $name ) === false)
		{
			if (! $use_dummy) return "";
			$name = get_property( "default_icon_name" );
		}

		// check: size		
		if ($sx == 0) $sx = $this->icon_sx;
		if ($sy == 0) $sy = $this->icon_sy;


		// build
		if (($sx > 0) && ($sy > 0))
		{
			$url = sprintf( "<img src='view_icon.php?ico=%s&amp;s=%d' width='%d' height='%d' border='0' alt='%s'>",
				$name, $sx, $sx, $sy, $name );
		}
		return $url;
	}

	/**
	 * print a HTML option list
	 * 
	 * @param		$ref_id			ID of selected icon
	 * @param		$excl_id		ID of icon to exclude from option list
	 * @param		$arg_name		name of HTML formular element
	 * @param		$default		name of default entry (un-used, if empty)
	 * @param		$form_name		name of HTML formular
	 * @param		$default_value	Value of default entry 
	 */
	public function html_options ( $ref_id, $excl_id, $arg_name, $default="", $form_name="", $default_value="" )
	{
		if (strlen($form_name)==0)
		{
			printf( "<select name='%s'>", $arg_name	);
		}
		else
		{
			printf( "<select name='%s' onchange='%s.submit()'>", $arg_name, $form_name );
		}
		
		// default
		if ((strlen($default)!=0) && ($default_value != $excl_id)) cat_option( $default_value, $ref_id, $default );
	
		foreach( $this->icons as $value )
		{
			if (strcmp($value, $excl_id)!=0) cat_option( $value, $ref_id, $value );
		}
		printf( "</select>"	);

		return;
	}

	/**
	 * print a HTML image-map with all icons
	 * 
	 * @param	$url_prefix		prefix of url, will be concatenated by icon-names
	 * @param	$default_text	text for default icon
	 * @param	$default_value	name of default icon
	 */
	public function html_chooser ( $url_prefix, $default_text="", $default_value="" )
	{
		// create preview, if necessary
		$this->create_preview();

		// printout preview as click map		
		printf( "<img src='%s?preview=1' border='0' usemap='#map_locations' alt='available icons'>", VIEW_ICON_PHP );
		$this->print_icon_map( $url_prefix, $default_text, $default_value );

		return;
	}

	/**
	 * clear all icon files from cache
	 *
	 * @param	$message		string for messages
	 * 
	 * @return	int				status
	 */
	public function clear_cache ( &$message )
	{
		$cache_dir = get_property( "cache_dir" );
		$cache_ico_prefix = get_property("cache_ico_prefix");
		$prefix_len = strlen( $cache_ico_prefix );
		$clear_count = 0;
	
		// check		
		if (! is_dir($cache_dir))
		{
			$message = get_string('s_CacheDirectory_dsc')." ($cache_dir) ".get_string('NotValid_dsc')."!";
			return STATUS_ERROR;
		}
		
		// open directory
	    if ($dh = opendir($cache_dir))
	    {
	        while (($file = readdir($dh)) !== false)
	        {
	        	if (strpos( $file, $cache_ico_prefix )===false) continue;
	        	if (strpos( $file, $cache_ico_prefix )!=0) continue;
	        	$file_name = $cache_dir . "/" . $file;
	        	if (is_file($file_name))
	        	{
	        		if (@unlink($file_name)) $clear_count++;
	        	}
			}
		    closedir($dh);
	    }
	    $message = get_string('ClearIconCacheCompleted_dsc').", $clear_count ".get_string('FilesDeleted_dsc')."!";
	    return STATUS_OK;
	}

	/**
	 * create names for the preview
	 * (png- and map-file)
	 */
	private function create_names()
	{
		global $running_script_base;
		
		// complete path
		if (strpos( php_uname(), "indows" ) === FALSE)
		{
			// on NON windows systems, the first char must be a slash
			if ($this->icon_dir[0]!='/')
			{
				$this->icon_full_dir = $running_script_base . "/" . $this->icon_dir;
			}
			else
			{
				$this->icon_full_dir = $this->icon_dir;
			}
		}
		else
		{
			// on windows systems the second char must be a colon
			if ($this->icon_dir[1]!=':')
			{
				$this->icon_full_dir = $running_script_base . "/" . $this->icon_dir;
			}
			else
			{
				$this->icon_full_dir = $this->icon_dir;
			}
		}

		// create name of preview png file
		$this->preview_png = sprintf( "%s/%s.png",
			get_property("cache_dir"), 
			get_property("cache_ico_prefix")
			);

		// create name of preview map file (serialized binary)
		$this->preview_map = sprintf( "%s/%s.ima",
			get_property("cache_dir"),
			get_property("cache_ico_prefix")
			);

		return;
	}

	/**
	 * read directory with icons and store all found icons
	 * in our internal array
	 */
	private function load ()
	{
		// check		
		if (! is_dir($this->icon_full_dir)) return;

		// open directory
		$this->set_icon_mtime( filemtime( $this->icon_full_dir ) );
	    if ($dh = opendir($this->icon_full_dir))
	    {
	        while (($file = readdir($dh)) !== false)
	        {
	        	$file_ext = strtolower( substr( $file, strrpos( $file, '.' ) ) );
	        	if (strcmp($file_ext, ".ico")==0)
	        	{
		        	$file_name = $this->icon_full_dir . "/" . $file;
		        	if (is_file($file_name))
		        	{
		        		$image_type = exif_imagetype( $file_name );
		        		$this->set_icon_mtime( filemtime( $file_name ) );
	//	        		if ($image_type == IMAGETYPE_GIF) $this->icons[] = $file;
		        		$this->icons[] = $file;
		        	}
	        	}
			}
		    closedir($dh);
	    }
	    
	    // sort by name
	    sort( $this->icons );
	       
	    return;
	}

	/**
	 * create preview with all icons from the list (png- and map-file)
	 * 
	 * this is done only, if no preview exists or some icon is newer
	 * than the preview
	 * 
	 */
	private function create_preview ()
	{
		// check preview files
		$this->set_preview_mtime();
		if ($this->icon_mtime < $this->preview_mtime) return;

		// get settings
		$padding	= get_property("map_padding");
		$sx			= get_property("map_sx");
		$sy			= get_property("map_sy");

		// reset icon_map array
		$this->reset_icon_map();

		// calc
		$icon_count = $this->count() + 1;
		
		$icon_fx	= $this->icon_sx + $padding;
		$icon_fy	= $this->icon_sy + $padding;
		$max_sx		= $sx - $padding; 
		$max_x		= (int)floor($max_sx / $icon_fx);
		$max_y		= (int)floor($icon_count / $max_x) + 1;
		$sy			= $max_y * $icon_fy + $padding;

		// create image resource
		$im = imagecreatetruecolor( $sx, $sy );
		$color_back = ImageColorAllocate ($im, 255, 255, 255);
		$color_red  = ImageColorAllocate ($im, 127,   0,   0);
		imagefill( $im, 0, 0, $color_back );
		
		// insert icons
		$px = $padding;
		$py = $padding;
		$px2 = $px + $this->icon_sx - 1;
		$py2 = $py + $this->icon_sy - 1;
		
		// default icon (empty one)
		imagerectangle( $im, $px, $py, $px2, $py2, $color_red );
		$this->add_icon_map( "", $px, $py, $px2, $py2 );
		
		// next pos
		$px = $px + $icon_fx;
		$px2 = $px + $this->icon_sx - 1;
		
		// icons from list
		foreach( $this->icons as $value )
		{
			$icon = $this->get_icon_image( $value, $this->icon_sx );
			imagecopy( $im, $icon, $px, $py, 0, 0, $this->icon_sx, $this->icon_sy );
			$this->add_icon_map( $value, $px, $py, $px2, $py2 );

			// next pos
			$px = $px + $icon_fx;
			$px2 = $px + $this->icon_sx - 1;
			if ($px2 >= $sx)
			{
				$px = $padding;
				$py = $py + $icon_fy;
				$px2 = $px + $this->icon_sx - 1;
				$py2 = $py + $this->icon_sy - 1;
			}
		}

		// save image
		ImagePng( $im, $this->preview_png );

		// save icon_map
		$this->save_icon_map();

		return;
	}

	/**
	 * store the given time in icon_mtime field, if
	 * it is smaller than the current value or current value is zero
	 * 
	 * @param	int		$_mtime
	 * @return	none
	 */
	private function set_icon_mtime( $_mtime )
	{
		if ($this->icon_mtime < $_mtime) $this->icon_mtime = $_mtime;

		return;
	}

	/**
	 * store the last modificatio time of preview files in preview_mtime field
	 * 
	 * @return	none
	 */
	private function set_preview_mtime()
	{
		if (! file_exists( $this->preview_png )) return;
		if (! file_exists( $this->preview_map )) return;
		
		$_mtime = filemtime( $this->preview_png );
		if ($this->preview_mtime < $_mtime) $this->preview_mtime = $_mtime;

		$_mtime = filemtime( $this->preview_map );
		if ($this->preview_mtime < $_mtime) $this->preview_mtime = $_mtime;

		return;
	}

	/**
	 * printout icon_map as HTML click map stream
	 * 
	 * @param	$url_prefix		prefix of url, will be concatenated by icon-names
	 * @param	$empty_text		alternate text of empty icon
	 * @param	$empty_name		name of empty icon
	 * @return	none
	 */
	private function print_icon_map ( $url_prefix, $empty_text="", $empty_name="" )
	{
		$this->load_icon_map();
		printf( "<map name='map_locations'>\n" );

		foreach( $this->icon_map as $data )
		{
			$_name = $_text = $data['name'];
			if (strlen($_name)==0)
			{
				$_name = $empty_name;
				$_text = $empty_text;
			}
			if (strlen($_text)!=0)
			{
				printf("<area shape='rect' coords='%d,%d,%d,%d' href='%s%s' alt='%s' title='%s'>\n",
					$data['x1'], $data['y1'],
					$data['x2'], $data['y2'],
					$url_prefix, $_name,
					$_text, $_text );
			}
		}

		printf( "</map>\n" );

		return;
	}

	/**
	 * reset icon_map
	 * 
	 * @return	none
	 */
	private function reset_icon_map ()
	{
		$this->icon_map = array();
		return;
	}

	/**
	 * store icon information in icon_map
	 * 
	 * @param	string		$_name		icon name
	 * @param	int			$_x1		left upper coord
	 * @param	int			$_y1		left upper coord
	 * @param	int			$_x2		right lower coord
	 * @param	int			$_y2		right lower coord
	 * @return	none
	 */
	private function add_icon_map ( $_name, $_x1, $_y1, $_x2, $_y2 )
	{
		$this->icon_map[] = array(
			'name' => $_name,
			'x1' => $_x1,
			'y1' => $_y1,
			'x2' => $_x2,
			'y2' => $_y2,
		);

		return;
	}

	/**
	 * load icon_map from file (unserialize)
	 * 
	 * @return	none
	 */
	private function load_icon_map ()
	{
		$handle = fopen ($this->preview_map, "r");
		$content = fread ($handle, filesize ($this->preview_map));
		fclose ($handle);
		$this->icon_map = unserialize( $content );		

		return;
	}

	/**
	 * store icon_mao to file (serialize)
	 */
	private function save_icon_map ()
	{
		$content = serialize( $this->icon_map );
		$handle = fopen ($this->preview_map, "w");
		$content = fwrite ($handle, $content );
		fclose ($handle);

		return;
	}

}
