<?php
# php-cmdb - a php based configuration management database

# Copyright (C) 2008  Dr. Michael Scheibe - michael@nofiles.de

# php-cmdb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# php-cmdb is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with php-cmdb.  If not, see <http://www.gnu.org/licenses/>.

/**
* @author		Dr. Michael Scheibe
* @package		cmdb
* @subpackage	processing
* @version		$Id: item_processor.inc.php 366 2009-04-16 15:40:32Z micha $
*/

require_once 'includes/entity_processor.inc.php';

/**
 * abstract class to manage an item with a list of attributes
 */
abstract class item_processor extends entity_processor
{
	/**
	 * list of attributes 
	 * @var	array
	 */
	protected $attr_list	= array();
	
	/**
	 * id of cloned item
	 * @var	integer
	 */
	protected $clone_id		= 0;

	/**
	 * overload attribute values from arguments
	 * 
	 * call load_from_args for all attributes
	 * 
	 * @return	none
	 */
	public function load_from_args()
	{
		if ($this->attr_count()!=0)
		{
			foreach ($this->attr_list as $attr)
			{
				$attr->load_from_args();
			}
		}
		return;
	}

	/**
	 * load attribute values from implicit functions
	 * 
	 * call update_func_values for all attributes
	 * 
	 * @return	none
	 */
	public function load_from_func()
	{
		$this->update_func_values();
		return;
	}

	/**
	 * load attribute values from array, delivered from CSV import
	 * 
	 * the array has ta_id => value mappings
	 * 
	 * @param	array	&$ta_val_map
	 * @return	none
	 */
	public function load_from_csv( &$ta_val_map )
	{
		if ($this->attr_count()!=0)
		{
			foreach ($this->attr_list as $attr)
			{
				$attr->load_from_csv( $ta_val_map );
			}
		}
		return;
	}

	/**
	 * store attribute values into csv array
	 * 
	 * @param	array	&$ta_val_map
	 * @return	none
	 */
	public function store_to_csv( &$ta_val_map )
	{
		if ($this->attr_count()!=0)
		{
			foreach ($this->attr_list as $attr)
			{
				$attr->store_to_csv( $ta_val_map );
			}
		}
		return;
	}

	/**
	 * create a clone of this object
	 * 
	 * @return	none
	 */
	public function __clone()
	{
		parent::__clone();
		$this->attr_list = $this->clone_attr_list();
	}

	// get

	/**
	 * reply classtype code
	 * 
	 * @return	integer
	 */
	abstract protected function get_classtype();

	/**
	 * reply item_type ID
	 * 
	 * @return	integer
	 */
	abstract public function get_item_type_id();

	/**
	 * reply ID of clone
	 * 
	 * @return	integer
	 */
	public function get_clone_id()
	{
		return $this->clone_id;
	}

	/**
	 * reply title of item, default is '-'
	 * 
	 * @return	string
	 */
	public function get_title()
	{
		$item_title = "-";
		return $item_title; 
	}

	/**
	 * reply titel of item_type, default is '-'
	 * 
	 * @return	string
	 */
	public function get_item_type_title()
	{
		$item_type_title = "-";
		return $item_type_title; 
	}

	/**
	 * reply count of attributes
	 * 
	 * @param	boolean	$available_only		count only available attributes
	 * @return	integer
	 */
	public function attr_count( $available_only=false )
	{
		if ($available_only)
		{
			$count = $this->countAttr();
		}
		else
		{
			$count = count($this->attr_list);
		}
		return $count;
	}

	/**
	 * fill labels in label_list with the accordingly values from 
	 * the own attributes, if their labels match and their values
	 * are not empty
	 * 
	 * @param	array	$label_list			list of labels
	 * @return	none
	 */
	public function fill_label_list( $label_list )
	{
		if ($this->attr_count()!=0)
		{
			foreach ($this->attr_list as $attr)
			{
				$attr->fill_label_list( $label_list );
			}
		}

		return;
	}

	// set

	/**
	 * set object title
	 * 
	 * @param	string	$_title
	 * @return	none
	 */
	abstract protected function set_title( $_title );


	/**
	 * set ID of clone
	 * 
	 * @param	integer	$_id
	 * @return	none
	 */
	public function set_clone_id( $_id )
	{
		$this->clone_id = $_id;
		return;
	}

	// display

	/**
	 * display attributes as form elements
	 * 
	 * @return	none
	 */
	protected function attr_display_form()
	{
		if ($this->attr_count()!=0)
		{
			foreach ($this->attr_list as $attr)
			{
				$attr->display_form();
			}
		}
		
		return;
	}

	/**
	 * display attributes
	 * 
	 * @param	integer	$attr_visible		level of visibility
	 * @return	none
	 */
	protected function attr_display( $attr_visible=10 )
	{
		if ($this->attr_count()!=0)
		{
			foreach ($this->attr_list as $attr)
			{
				if ($attr->is_visible( $attr_visible ))
				{
					$attr->display();
				}
			}
		}

		return;
	}

	/**
	 * display attributes as row in a list
	 * 
	 * @param	integer	$cols
	 * @param	integer	$pre_cols
	 * @param	integer	$post_cols
	 * @return	none
	 */
	public function attr_list( $cols, $pre_cols=0, $post_cols=0 )
	{
		$size = $cols - $pre_cols - $post_cols;

		html_table_row();
		if ($pre_cols!=0) printf( "<td colspan='%d'></td>", $pre_cols );
		
		printf( "<td colspan='%d' class='c_search'>", $size );
		if ($this->attr_count()!=0)
		{
			foreach ($this->attr_list as $attr)
			{
				$attr->display_list();
			}
		}
		printf( "</td>" );

		if ($post_cols!=0) printf( "<td colspan='%d'></td>", $post_cols );
		printf( "</tr>\n" );

		return;
	}

	// action methoden

	/**
	 * insert object into database
	 *
	 * @return	boolean		status
	 */
	public function action_insert()
	{
		// myself first, need ID for my attributes
		$this->update_title();
		parent::action_insert();
		if ($this->get_status() != STATUS_OK) return false;

		// attributes
		if ($this->attr_count()!=0)
		{
			foreach ($this->attr_list as $attr)
			{
				$attr->set_classtype( $this->get_classtype() );
				$attr->set_classtype_id( $this->db_object->get_id() );
				if (! $attr->action_insert())
				{
					// get message and status
					$this->add_message( $attr->get_last_message(), $attr->get_status() );
				}
			}
		}

		// make value changes by implicit functions and update
		if ($this->get_status() == STATUS_OK)
		{
			$this->update_func_values();
		}

		// check status
		if ($this->get_status() != STATUS_OK)
		{
			// rollback
			$this->action_delete();
		}
		else
		{
			// create my title
			$this->update_title();
			parent::action_update();
		}

		// final status
		$rc = ($this->get_status() == STATUS_OK);

		return $rc;
	}

	/**
	 * update database from object content
	 * 
	 * @return	boolean		status 
	 */
	public function action_update()
	{
		$modify_counter = 0;
		
		// update attributes
		if ($this->attr_count()!=0)
		{
			foreach ($this->attr_list as $attr)
			{
				$modify_counter += $attr->is_modified();
				if ($attr->get_id() == 0)
				{
					// new attribute: insert
					$attr->set_classtype( $this->get_classtype() );
					$attr->set_classtype_id( $this->db_object->get_id() );
					if (! $attr->action_insert())
					{
						// get message and status
						$this->add_message( $attr->get_last_message(), $attr->get_status() );
					}				
				}
				else
				{
					// available attribute: update
					if (! $attr->action_update())
					{
						// get message and status
						$this->add_message( $attr->get_last_message(), $attr->get_status() );
					}
				}
			}
		}

		// make value changes by implicit functions and update
		if ($this->get_status() == STATUS_OK)
		{
			$this->update_func_values();
		}

		// update item, if attribute changes happens
		if (($modify_counter != 0) && ($this->get_status() == STATUS_OK))
		{ 
			$this->update_title();
			$this->db_object->activate_modify();
		}		

		// run parent stuff 
		if ($this->get_status() == STATUS_OK) parent::action_update();

		return ($this->get_status() == STATUS_OK);
	}

	/**
	 * delete object content from database
	 * 
	 * @return	boolean		status 
	 */
	public function action_delete()
	{
		// attributes first
		if ($this->attr_count()!=0)
		{
			foreach ($this->attr_list as $attr)
			{
				if (! $attr->action_delete())
				{
					// get message and status
					$this->add_message( $attr->get_last_message(), $attr->get_status() );
				}
			}
		}

		// delete item itself
		if ($this->get_status() == STATUS_OK)
		{
			parent::action_delete();
		}	

		return ($this->get_status() == STATUS_OK);
	}

	// helper

	/**
	 * run get_input_value and update for all attributes with ATTR_INPUT_FUNC
	 * 
	 * @return	none
	 */
	protected function update_func_values()
	{
		if ($this->attr_count()!=0)
		{
			foreach ($this->attr_list as $attr)
			{
				$attr->load_from_func( $this );

				if ($attr->get_id() == 0)
				{
					$attr->set_classtype( $this->get_classtype() );
					$attr->set_classtype_id( $this->db_object->get_id() );
					if (! $attr->action_insert())
					{
						// get message and status
						$this->add_message( $attr->get_last_message(), $attr->get_status() );
					}
				}
				else
				{
					if (! $attr->action_update())
					{
						// get message and status
						$this->add_message( $attr->get_last_message(), $attr->get_status() );
					}
				}
			}
		}

		return;
	}

	/**
	 * create/update title from item attributes
	 * 
	 * @return	none
	 */
	private function update_title()
	{
		$_title = "";
		if ($this->attr_count()!=0)
		{
			foreach ($this->attr_list as $attr)
			{
				$tag_part = $attr->get_tag_part();
				if (strlen($tag_part)>0)
				{
					if (strlen($_title)) $_title = $_title . " ";
					$_title = $_title . $tag_part;
				}
			}
		}
		
		// check: title empty
		if (strlen($_title)==0) $_title = sprintf("SYS-ID: %03d-%d", $this->get_classtype(), $this->db_object->get_id() );
		
		// set
		$this->set_title( $_title );
		
		return;
	}

	/**
	 * fill $this->attr_list with new attribute-objects
	 * in the right order
	 * 
	 * @return	none
	 */
	protected function create_attr_list()
	{
		$item_id  = $this->get_id();
		$item_type_id = $this->get_item_type_id();
		$ct = $this->get_classtype();

		// could not do anything without an item_type_id 
		if ($item_type_id == 0) return;

		// build sql statement
		$db     = core_db::getInstance();
		if ($item_id == 0)
		{
			$query  = "SELECT * FROM type_attr " .
				"WHERE ta_classtype='$ct' " .
				"AND ta_classtype_id='$item_type_id' " .
				"ORDER BY ta_order";
		}
		else
		{
			$query = "select * from type_attr " .
				"left join attr on a_ta_id=ta_id " .
				"and ((a_classtype IS NULL) or (a_classtype='$ct')) " .
				"and ((a_classtype_id IS NULL) or (a_classtype_id='$item_id')) " .
				"where ta_classtype='$ct' " .
				"and ta_classtype_id='$item_type_id' " .
				"order by ta_order";
		}
//echo "DEBUG: $query<br>\n";
		$result = $db->query( $query );
		$num = $db->num_rows($result);
		for ($i=0; $i<$num; $i++)
		{
			$data = $db->fetch( $result );
//print_r( $data );
			if (! isset($data['a_id']))
			{
				$data['a_id']=0;
				$data['a_ta_id']=$data['ta_id'];
			}
			$this->attr_list[ $i ] = & create_attr( $data );
		}
		return;
	}


	/**
	 * clone attributes 
	 * 
	 * @return	array
	 */
	private function clone_attr_list()
	{
		$attr_list_clone = array();
		if ($this->attr_count()!=0)
		{
			foreach ($this->attr_list as $attr)
			{
				$attr_list_clone[] = clone $attr;
			}
		}
		return $attr_list_clone;		
	}

	/**
	 * returns number of attributes defined by type
	 * 
	 * @return	int
	 */
	protected function countAttrByType()
	{
		$db     = core_db::getInstance();
		$query  = "SELECT COUNT(*) FROM `type_attr` " .
			"WHERE `ta_classtype`=" . $this->get_classtype() . " " .
			"AND `ta_classtype_id`=" . $this->get_item_type_id();
		$result	= $db->query( $query );
		$count	= $db->result( $result, 0 );
		return $count;
	}

	/**
	 * returns actually number of attributes of this item
	 * found in database
	 * 
	 * @return	int
	 */
	protected function countAttr()
	{
		$db     = core_db::getInstance();
		$query  = "SELECT COUNT(*) FROM `attr` " .
			"WHERE `a_classtype`=" . $this->get_classtype() . " " .
			"AND `a_classtype_id`=" . $this->db_object->get_id();
		$result	= $db->query( $query );
		$count	= $db->result( $result, 0 );
		return $count;
	}

}