<?php
# php-cmdb - a php based configuration management database

# Copyright (C) 2008  Dr. Michael Scheibe - michael@nofiles.de

# php-cmdb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# php-cmdb is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with php-cmdb.  If not, see <http://www.gnu.org/licenses/>.

require_once 'tables/db_user.inc.php';
require_once 'includes/page_error.inc.php';
require_once 'includes/user.inc.php';
require_once 'includes/user_list.inc.php';
require_once 'includes/ldap_api.inc.php';

/**
 * try to import user data from ldap directory
 * 
 * @param	string	$login_name			users login name
 * @return	none
 */
function importUserFromLDAP( $login_name )
{
	// query data from ldap
	$ldap_info = ldap_api_check_uid( $login_name );
	$dn = ldap_api_get_dn( $ldap_info );
	
	// check group
	if ((strlen($dn)!=0) && (ldap_api_check_group( $dn )))
	{
		// auto import
		$surname = 'SURNAME';
		$givenname = 'GIVENNAME';
		$comman_name = array('SURNAME', 'GIVENNAME' );
		
		if (isset($ldap_info['cn'][0]))
		{
			$common_name = explode(',', $ldap_info['cn'][0]);
		}
		if (isset($ldap_info['sn'][0]))
		{
			$surname = utf8_decode( $ldap_info['sn'][0] );
		}
		else
		{
			$surname = utf8_decode( $common_name[0] );
		}
		if (isset($ldap_info['givenname'][0]))
		{
			$givenname = utf8_decode( $ldap_info['givenname'][0] );
		}
		else
		{
			$givenname = utf8_decode( $common_name[1] );
		}

		// create user
		$import_user = new user( 0 );
		
		// set attributes from ldap_info 
		$_POST['u_role_id']=ROLE_VIEWER;
		$_POST['u_auth_backend']=AUTH_BACKEND_LDAP;
		$_POST['u_active']=1;
		$_POST['u_login']=$login_name;
		$_POST['u_last_name']=trim( $surname );
		$_POST['u_first_name']=trim( $givenname );
		if (isset($ldap_info['mail'][0])) $_POST['u_email']=$ldap_info['mail'][0];

		$_POST['u_form']=1;
		$import_user->load_from_args();
		$_POST['u_form']=2;
		$import_user->load_from_args();

		// store in out database
		$import_user->action_insert();
	}
	
	return;	
}

// load args
$login_form	= http_get_string( 'l_form', '' );
$login_name	= http_get_string( 'l_name', '' );
$login_pass	= http_get_passwd( 'l_pass', '' );

if (($login_form == core_session::getInstance()->get_session_id()))
{
	// process login data
	$_id = user_list::get_by_login( $login_name );

	// if user not found but ldap and ldap_auto_import are configured,
	// try to get it from the ldap directory 
	if (
		($_id == 0) &&
		(get_property('auth_ldap')) &&
		(get_property('auth_ldap_auto_import'))
		)
	{
		$_id = importUserFromLDAP( $login_name );
		
		// now the data should be imported, so we try login again
		$_id = user_list::get_by_login( $login_name );
	}
	
	// check: user available
	if ($_id == 0)
	{
		// login not found
		html_login_error_page( 'LoginErrUser_inf' );
		exit();
	}

	// check: empty password
	if ( (strlen($login_pass)==0) && ! get_property('empty_passwd') )
	{
		// empty password not allowed
		html_login_error_page( 'LoginErrEmtyPass_inf' );
		exit();
	}

	// login found, load user data
	$user = new db_user( $_id );

	// check: active status 
	if (! $user->is_active())
	{
		html_login_error_page( 'LoginErrUserActive_inf' );
		exit();
	}

	// check: password
	// using configured auth_backend
	switch ($user->get_auth_backend())
	{
		case AUTH_BACKEND_LOCAL:
		{
			// check: password
			if (! $user->password_match( $login_pass) )
			{
				// wrong password
				html_login_error_page( 'LoginErrPassword_inf' );
				exit();
			}
			break;
		}
		case AUTH_BACKEND_LDAP:
		{
			$ldap_info = ldap_api_check_uid( $user->get_login() );
			$dn = ldap_api_get_dn( $ldap_info );
			if (strlen($dn)==0)
			{
				// user not found
				html_login_error_page( 'LoginErrUser_inf' );
				exit();
			}

			// check: group
			if (!ldap_api_check_group($dn))
			{
				// user not in group
				html_login_error_page( 'LoginErrUserGroup_inf' );
				exit();
			}

			// check: empty password
			if (strlen($login_pass)==0)
			{
				// password empty
				html_login_error_page( 'LoginErrEmtyPass_inf' );
				exit();
			}
		
			// check: password
			if (! ldap_api_check_passwd( $dn, $login_pass ))
			{
				// wrong password
				html_login_error_page( 'LoginErrPassword_inf' );
				exit();
			}

			// if role-mapping by LDAP is configured, we get it now 
			// and store it in the users record
			$role_id = ldap_api_get_role( $dn, $user->get_role_id() );
			$user->set_role_id( $role_id );
			$user->db_update();

			break;
		}
		default:
		{
			html_login_error_page( 'LoginErrBackEnd' );
			exit();
		}
	}

	// authentication completed: login and passwort are validated
	
	// load session data 
	core_session::getInstance()->set_user_id( $_id );
	core_session::getInstance()->set_user_name( $user->get_name() );
	core_session::getInstance()->set_role_id( $user->get_role_id() );
	core_session::getInstance()->set_role_name( get_enum_to_string('enum_roles', $user->get_role_id() ) );

	// store session
	core_session::getInstance()->store();
	
}
else
{
	// no login data found, display login form
	include_once 'includes/login_form.inc.php';
	exit();
}
