<?php
# php-cmdb - a php based configuration management database

# Copyright (C) 2008  Dr. Michael Scheibe - michael@nofiles.de

# php-cmdb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# php-cmdb is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with php-cmdb.  If not, see <http://www.gnu.org/licenses/>.

/**
 * base class to include optional modules
 */
abstract class module_base
{
	/**
	 * replays an enum-string with supported labels
	 * of this module
	 * 
	 * note: enum string means a comma separated list of
	 * key-value pairs like this:
	 * 
	 * "1:one,2:two,3:three"
	 * 
	 * where key is a numeric value to identify a label and the
	 * value is the presented name of a label
	 * 
	 * @return	string
	 */
	abstract public function get_enum_label_string();

	/**
	 * replays an array with names and descriptions
	 * of display classes, supported by this module
	 * 
	 * display classes must be childs from relation_peer_imx class
	 * 
	 * Result-Array: ['class-name']='description'
	 * 
	 * @param	$ci				CI
	 * @return	array			name of display class
	 */
	abstract public function get_display_class( $ci );
	
	/**
	 * replays a value for an attribute with the given label inside
	 * the given item_processor object
	 * 
	 * @param	int		$label				label of attribute
	 * @param	object	$item_processor		object, where the attribute belongs to
	 * @return	mixed
	 */
	abstract public function get_input_value( $label, &$item_processor );

}

/**
 * singleton class as central interface to optional modules
 */
class module_list
{
	/**
	 * array of registered modules
	 * 
	 * @var	array
	 */
	static protected $modules = array();

	/**
	 * constructor
	 * 
	 * @return	none
	 */
	protected function __construct()
	{
		return;
	}

	/**
	 * replays count of registered modules
	 * 
	 * @return	int
	 */
	static public function count()
	{
		return count( self::$modules );
	}

	/**
	 * register a module
	 * 
	 * @param	object	$module				modul object
	 * @return	none
	 */
	static public function register( $module )
	{
//		if (! class($module)) return;
		if (! is_subclass_of( $module, "module_base" )) return;
		self::$modules[] = $module;
		return;
	}

	/**
	 * replays an enum-string with supported labels
	 * of all modules
	 * 
	 * @return	string
	 */
	static public function get_enum_label_string()
	{
		$enum_label_string = "0:---";
		foreach (self::$modules as &$module)
		{
			$module_enum_label_string = $module->get_enum_label_string();
			if (strlen($module_enum_label_string)>0)
			{
				$enum_label_string = $enum_label_string . "," . $module_enum_label_string;
			}
		}
		return $enum_label_string;
	}

	/**
	 * replays an array with names and descriptions
	 * of display classes, supported by all modules for the given ci
	 * 
	 * Result-Array: ['class-name']='description'
	 * 
	 * @param	object	$ci				CI object
	 * @return	array
	 */
	static public function get_display_class( $ci )
	{
		$class_array = array();
		$class_array[CI_MAP_CLASS_DEFAULT]="Standard View"; 
		foreach (self::$modules as &$module)
		{
			$module_class_array = $module->get_display_class( $ci );
			foreach( $module_class_array as $class_name => $class_desc )
			{
				$class_array[ $class_name ] = $class_desc;
			}
		}
//print_r( $class_array );
		return $class_array;
	}

	/**
	 * get value for the given label
	 * 
	 * @param	int		$label				label of attribute
	 * @param	object	$item_processor		object, where the attribute belongs to
	 * @return	mixed
	 */
	static public function get_input_value( $label, &$item_processor )
	{
		$pre_value = "";
		foreach (self::$modules as &$module)
		{
			if (strlen($pre_value)==0) $pre_value = $module->get_input_value( $label, $item_processor );
		}
//print_r( $class_array );
		return $pre_value;
	}
	
}
